@@ -101,10 +101,10 @@ extern void __cfs_free_pages(cfs_page_t *page, unsigned int order);
  * SLAB allocator
  * XXX Liang: move these declare to public file
  */
-#ifdef HAVE_KMEM_CACHE_S
-typedef struct kmem_cache_s cfs_mem_cache_t;
-#else
+#ifdef HAVE_KMEM_CACHE
 typedef struct kmem_cache cfs_mem_cache_t;
+#else
+typedef kmem_cache_t cfs_mem_cache_t;
 #endif
 extern cfs_mem_cache_t * cfs_mem_cache_create (const char *, size_t, size_t, unsigned long);
 extern int cfs_mem_cache_destroy ( cfs_mem_cache_t * );
