@@ -145,7 +145,8 @@ typedef struct gmnal_tx {
                 struct iovec    *iov;           /* mapped frags */
                 lnet_kiov_t     *kiov;          /* page frags */
         }                        tx_large_frags;
-        unsigned long            tx_launchtime; /* when (in jiffies) the transmit was launched */
+        cfs_time_t               tx_launchtime; /* when (in jiffies) the
+                                                 * transmit was launched */
         struct gmnal_tx         *tx_next;       /* stash on gmni_txs */
 } gmnal_tx_t;
 
@@ -175,7 +176,7 @@ typedef struct gmnal_ni {
         gmnal_tx_t       *gmni_txs;             /* all txs */
         gmnal_rx_t       *gmni_rxs;		/* all rx descs */
         gmnal_txbuf_t    *gmni_ltxbs;           /* all large tx bufs */
-        
+
         atomic_t          gmni_nthreads;        /* total # threads */
         gm_alarm_t        gmni_alarm;           /* alarm to wake caretaker */
         int               gmni_shutdown;	/* tell all threads to exit */
