@@ -847,6 +847,10 @@ kibnal_add_persistent_peer (lnet_nid_t nid)
 
         write_lock_irqsave (&kibnal_data.kib_global_lock, flags);
 
+        /* I'm always called with a reference on kibnal_data.kib_ni
+         * so shutdown can't have started */
+        LASSERT (kibnal_data.kib_listener_cep != NULL);
+
         peer2 = kibnal_find_peer_locked (nid);
         if (peer2 != NULL) {
                 kibnal_peer_decref (peer);
@@ -2128,7 +2132,8 @@ kibnal_module_init (void)
         int    rc;
 
         if (the_lnet.ln_ptlcompat != 0) {
-                LCONSOLE_ERROR("IIB does not support portals compatibility mode\n");
+                LCONSOLE_ERROR_MSG(0x12c, "IIB does not support portals "
+                                   "compatibility mode\n");
                 return -ENODEV;
         }
         
