@@ -248,7 +248,7 @@ typedef struct ksock_conn
         void               *ksnc_saved_write_space; /* socket's original write_space() callback */
         atomic_t            ksnc_conn_refcount; /* conn refcount */
         atomic_t            ksnc_sock_refcount; /* sock refcount */
-        ksock_sched_t	   *ksnc_scheduler;     /* who schedules this connection */
+        ksock_sched_t      *ksnc_scheduler;     /* who schedules this connection */
         __u32               ksnc_myipaddr;      /* my IP */
         __u32               ksnc_ipaddr;        /* peer's IP */
         int                 ksnc_port;          /* peer's port */
@@ -290,7 +290,7 @@ typedef struct ksock_conn
         atomic_t            ksnc_tx_nob;        /* # bytes queued */
         int                 ksnc_tx_ready;      /* write space */
         int                 ksnc_tx_scheduled;  /* being progressed */
-        
+
 #if !SOCKNAL_SINGLE_FRAG_RX
         struct iovec        ksnc_rx_scratch_iov[LNET_MAX_IOV];
 #endif
@@ -367,12 +367,16 @@ extern ksock_proto_t ksocknal_protocol_v2x;
 #define KSOCK_PROTO_V1_MINOR    LNET_PROTO_TCP_VERSION_MINOR
 #define KSOCK_PROTO_V1          KSOCK_PROTO_V1_MAJOR
 
+#ifndef CPU_MASK_NONE
+#define CPU_MASK_NONE   0UL
+#endif
+
 static inline int
-ksocknal_route_mask(void) 
+ksocknal_route_mask(void)
 {
         if (!*ksocknal_tunables.ksnd_typed_conns)
                 return (1 << SOCKLND_CONN_ANY);
-        
+
         return ((1 << SOCKLND_CONN_CONTROL) |
                 (1 << SOCKLND_CONN_BULK_IN) |
                 (1 << SOCKLND_CONN_BULK_OUT));
