@@ -4,105 +4,218 @@
 
 #include "socklnd.h"
 
-# if CONFIG_SYSCTL && !CFS_SYSFS_MODULE_PARM
-static ctl_table ksocknal_ctl_table[21];
+# if defined(CONFIG_SYSCTL) && !CFS_SYSFS_MODULE_PARM
+static cfs_sysctl_table_t ksocknal_ctl_table[21];
 
-ctl_table ksocknal_top_ctl_table[] = {
-        {200, "socknal", NULL, 0, 0555, ksocknal_ctl_table},
+cfs_sysctl_table_t ksocknal_top_ctl_table[] = {
+        {
+                .ctl_name = 200,
+                .procname = "socknal",
+                .data     = NULL,
+                .maxlen   = 0,
+                .mode     = 0555,
+                .child    = ksocknal_ctl_table
+        },
         { 0 }
 };
 
 int
 ksocknal_lib_tunables_init ()
 {
-	int    i = 0;
-	int    j = 1;
+        int    i = 0;
+        int    j = 1;
 
-        ksocknal_ctl_table[i++] = (ctl_table)
-		{j++, "timeout", ksocknal_tunables.ksnd_timeout,
-		 sizeof (int), 0644, NULL, &proc_dointvec};
-        ksocknal_ctl_table[i++] = (ctl_table)
-		{j++, "credits", ksocknal_tunables.ksnd_credits,
-		 sizeof (int), 0444, NULL, &proc_dointvec};
-        ksocknal_ctl_table[i++] = (ctl_table)
-		{j++, "peer_credits", ksocknal_tunables.ksnd_peercredits,
-		 sizeof (int), 0444, NULL, &proc_dointvec};
-        ksocknal_ctl_table[i++] = (ctl_table)
-		{j++, "nconnds", ksocknal_tunables.ksnd_nconnds,
-		 sizeof (int), 0444, NULL, &proc_dointvec};
-        ksocknal_ctl_table[i++] = (ctl_table)
-		{j++, "min_reconnectms", ksocknal_tunables.ksnd_min_reconnectms,
-		 sizeof (int), 0444, NULL, &proc_dointvec};
-        ksocknal_ctl_table[i++] = (ctl_table)
-		{j++, "max_reconnectms", ksocknal_tunables.ksnd_max_reconnectms,
-		 sizeof (int), 0444, NULL, &proc_dointvec};
-        ksocknal_ctl_table[i++] = (ctl_table)
-		{j++, "eager_ack", ksocknal_tunables.ksnd_eager_ack,
-		 sizeof (int), 0644, NULL, &proc_dointvec};
-        ksocknal_ctl_table[i++] = (ctl_table)
-		{j++, "zero_copy", ksocknal_tunables.ksnd_zc_min_frag,
-		 sizeof (int), 0644, NULL, &proc_dointvec};
-        ksocknal_ctl_table[i++] = (ctl_table)
-		{j++, "typed", ksocknal_tunables.ksnd_typed_conns,
-		 sizeof (int), 0444, NULL, &proc_dointvec};
-        ksocknal_ctl_table[i++] = (ctl_table)
-		{j++, "min_bulk", ksocknal_tunables.ksnd_min_bulk,
-		 sizeof (int), 0644, NULL, &proc_dointvec};
-        ksocknal_ctl_table[i++] = (ctl_table)
-		{j++, "rx_buffer_size", ksocknal_tunables.ksnd_rx_buffer_size,
-		 sizeof(int), 0644, NULL, &proc_dointvec};
-        ksocknal_ctl_table[i++] = (ctl_table)
-		{j++, "tx_buffer_size", ksocknal_tunables.ksnd_tx_buffer_size,
-		 sizeof(int), 0644, NULL, &proc_dointvec};
-        ksocknal_ctl_table[i++] = (ctl_table)
-		{j++, "nagle", ksocknal_tunables.ksnd_nagle,
-		 sizeof(int), 0644, NULL, &proc_dointvec};
-#if CPU_AFFINITY
-        ksocknal_ctl_table[i++] = (ctl_table)
-		{j++, "irq_affinity", ksocknal_tunables.ksnd_irq_affinity,
-		 sizeof(int), 0644, NULL, &proc_dointvec};
+        ksocknal_ctl_table[i++] = (cfs_sysctl_table_t) {
+                .ctl_name = j++,
+                .procname = "timeout",
+                .data     = ksocknal_tunables.ksnd_timeout,
+                .maxlen   = sizeof (int),
+                .mode     = 0644,
+                .proc_handler = &proc_dointvec
+        };
+        ksocknal_ctl_table[i++] = (cfs_sysctl_table_t) {
+                .ctl_name = j++,
+                .procname = "credits",
+                .data     = ksocknal_tunables.ksnd_credits,
+                .maxlen   = sizeof (int),
+                .mode     = 0444,
+                .proc_handler = &proc_dointvec
+        };
+        ksocknal_ctl_table[i++] = (cfs_sysctl_table_t) {
+                .ctl_name = j++,
+                .procname = "peer_credits",
+                .data     = ksocknal_tunables.ksnd_peercredits,
+                .maxlen   = sizeof (int),
+                .mode     = 0444,
+                .proc_handler = &proc_dointvec
+        };
+        ksocknal_ctl_table[i++] = (cfs_sysctl_table_t) {
+                .ctl_name = j++,
+                .procname = "nconnds",
+                .data     = ksocknal_tunables.ksnd_nconnds,
+                .maxlen   = sizeof (int),
+                .mode     = 0444,
+                .proc_handler = &proc_dointvec
+        };
+        ksocknal_ctl_table[i++] = (cfs_sysctl_table_t) {
+                .ctl_name = j++,
+                .procname = "min_reconnectms",
+                .data     = ksocknal_tunables.ksnd_min_reconnectms,
+                .maxlen   = sizeof (int),
+                .mode     = 0444,
+                .proc_handler = &proc_dointvec
+        };
+        ksocknal_ctl_table[i++] = (cfs_sysctl_table_t) {
+                .ctl_name = j++,
+                .procname = "max_reconnectms",
+                .data     = ksocknal_tunables.ksnd_max_reconnectms,
+                .maxlen   = sizeof (int),
+                .mode     = 0444,
+                .proc_handler = &proc_dointvec
+        };
+        ksocknal_ctl_table[i++] = (cfs_sysctl_table_t) {
+                .ctl_name = j++,
+                .procname = "eager_ack",
+                .data     = ksocknal_tunables.ksnd_eager_ack,
+                .maxlen   = sizeof (int),
+                .mode     = 0644,
+                .proc_handler = &proc_dointvec
+        };
+        ksocknal_ctl_table[i++] = (cfs_sysctl_table_t) {
+                .ctl_name = j++,
+                .procname = "zero_copy",
+                .data     = ksocknal_tunables.ksnd_zc_min_frag,
+                .maxlen   = sizeof (int),
+                .mode     = 0644,
+                .proc_handler = &proc_dointvec
+        };
+        ksocknal_ctl_table[i++] = (cfs_sysctl_table_t) {
+                .ctl_name = j++,
+                .procname = "typed",
+                .data     = ksocknal_tunables.ksnd_typed_conns,
+                .maxlen   = sizeof (int),
+                .mode     = 0444,
+                .proc_handler = &proc_dointvec
+        };
+        ksocknal_ctl_table[i++] = (cfs_sysctl_table_t) {
+                .ctl_name = j++,
+                .procname = "min_bulk",
+                .data     = ksocknal_tunables.ksnd_min_bulk,
+                .maxlen   = sizeof (int),
+                .mode     = 0644,
+                .proc_handler = &proc_dointvec
+        };
+        ksocknal_ctl_table[i++] = (cfs_sysctl_table_t) {
+                .ctl_name = j++,
+                .procname = "rx_buffer_size",
+                .data     = ksocknal_tunables.ksnd_rx_buffer_size,
+                .maxlen   = sizeof(int),
+                .mode     = 0644,
+                .proc_handler = &proc_dointvec
+        };
+        ksocknal_ctl_table[i++] = (cfs_sysctl_table_t) {
+                .ctl_name = j++,
+                .procname = "tx_buffer_size",
+                .data     = ksocknal_tunables.ksnd_tx_buffer_size,
+                .maxlen   = sizeof(int),
+                .mode     = 0644,
+                .proc_handler = &proc_dointvec
+        };
+        ksocknal_ctl_table[i++] = (cfs_sysctl_table_t) {
+                .ctl_name = j++,
+                .procname = "nagle",
+                .data     = ksocknal_tunables.ksnd_nagle,
+                .maxlen   = sizeof(int),
+                .mode     = 0644,
+                .proc_handler = &proc_dointvec
+        };
+#ifdef CPU_AFFINITY
+        ksocknal_ctl_table[i++] = (cfs_sysctl_table_t) {
+                .ctl_name = j++,
+                .procname = "irq_affinity",
+                .data     = ksocknal_tunables.ksnd_irq_affinity,
+                .maxlen   = sizeof(int),
+                .mode     = 0644,
+                .proc_handler = &proc_dointvec
+        };
 #endif
-        ksocknal_ctl_table[i++] = (ctl_table)
-		{j++, "keepalive_idle", ksocknal_tunables.ksnd_keepalive_idle,
-		 sizeof(int), 0644, NULL, &proc_dointvec};
-        ksocknal_ctl_table[i++] = (ctl_table)
-		{j++, "keepalive_count", ksocknal_tunables.ksnd_keepalive_count,
-		 sizeof(int), 0644, NULL, &proc_dointvec};
-	ksocknal_ctl_table[i++] = (ctl_table)
-		{j++, "keepalive_intvl", ksocknal_tunables.ksnd_keepalive_intvl,
-		 sizeof(int), 0644, NULL, &proc_dointvec};
+        ksocknal_ctl_table[i++] = (cfs_sysctl_table_t) {
+                .ctl_name = j++,
+                .procname = "keepalive_idle",
+                .data     = ksocknal_tunables.ksnd_keepalive_idle,
+                .maxlen   = sizeof(int),
+                .mode     = 0644,
+                .proc_handler = &proc_dointvec
+        };
+        ksocknal_ctl_table[i++] = (cfs_sysctl_table_t) {
+                .ctl_name = j++,
+                .procname = "keepalive_count",
+                .data     = ksocknal_tunables.ksnd_keepalive_count,
+                .maxlen   = sizeof(int),
+                .mode     = 0644,
+                .proc_handler = &proc_dointvec
+        };
+        ksocknal_ctl_table[i++] = (cfs_sysctl_table_t) {
+                .ctl_name = j++,
+                .procname = "keepalive_intvl",
+                .data     = ksocknal_tunables.ksnd_keepalive_intvl,
+                .maxlen   = sizeof(int),
+                .mode     = 0644,
+                .proc_handler = &proc_dointvec
+        };
 #ifdef SOCKNAL_BACKOFF
-        ksocknal_ctl_table[i++] = (ctl_table)
-                {j++, "backoff_init", ksocknal_tunables.ksnd_backoff_init,
-		 sizeof(int), 0644, NULL, &proc_dointvec};
-        ksocknal_ctl_table[i++] = (ctl_table)
-                {j++, "backoff_max", ksocknal_tunables.ksnd_backoff_max,
-		 sizeof(int), 0644, NULL, &proc_dointvec};
+        ksocknal_ctl_table[i++] = (cfs_sysctl_table_t) {
+                .ctl_name = j++,
+                .procname = "backoff_init",
+                .data     = ksocknal_tunables.ksnd_backoff_init,
+                .maxlen   = sizeof(int),
+                .mode     = 0644,
+                .proc_handler = &proc_dointvec
+        };
+        ksocknal_ctl_table[i++] = (cfs_sysctl_table_t) {
+                .ctl_name = j++,
+                .procname = "backoff_max",
+                .data     = ksocknal_tunables.ksnd_backoff_max,
+                .maxlen   = sizeof(int),
+                .mode     = 0644,
+                .proc_handler = &proc_dointvec
+        };
 #endif
+#if SOCKNAL_VERSION_DEBUG
+        ksocknal_ctl_table[i++] = (cfs_sysctl_table_t) {
+                .ctl_name = j++,
+                .procname = "protocol",
+                .data     = ksocknal_tunables.ksnd_protocol,
+                .maxlen   = sizeof(int),
+                .mode     = 0644,
+                .proc_handler = &proc_dointvec
+        };
+#endif
+        ksocknal_ctl_table[i++] =  (cfs_sysctl_table_t) { 0 };
 
-	LASSERT (j == i+1);
-	LASSERT (i < sizeof(ksocknal_ctl_table)/sizeof(ksocknal_ctl_table[0]));
+        LASSERT (j == i);
+        LASSERT (i <= sizeof(ksocknal_ctl_table)/sizeof(ksocknal_ctl_table[0]));
 
         ksocknal_tunables.ksnd_sysctl =
-                register_sysctl_table(ksocknal_top_ctl_table, 0);
+                cfs_register_sysctl_table(ksocknal_top_ctl_table, 0);
 
         if (ksocknal_tunables.ksnd_sysctl == NULL)
-		CWARN("Can't setup /proc tunables\n");
+                CWARN("Can't setup /proc tunables\n");
 
-	return 0;
+        return 0;
 }
 
 void
 ksocknal_lib_tunables_fini ()
 {
         if (ksocknal_tunables.ksnd_sysctl != NULL)
-                unregister_sysctl_table(ksocknal_tunables.ksnd_sysctl);
+                cfs_unregister_sysctl_table(ksocknal_tunables.ksnd_sysctl);
 }
 #else
 int
 ksocknal_lib_tunables_init ()
 {
-	return 0;
+        return 0;
 }
 
 void
@@ -114,7 +227,7 @@ ksocknal_lib_tunables_fini ()
 void
 ksocknal_lib_bind_irq (unsigned int irq)
 {
-#if (defined(CONFIG_SMP) && CPU_AFFINITY)
+#if (defined(CONFIG_SMP) && defined(CPU_AFFINITY))
         int              bind;
         int              cpu;
         char             cmdline[64];
@@ -149,7 +262,7 @@ ksocknal_lib_bind_irq (unsigned int irq)
                   "echo %d > /proc/irq/%u/smp_affinity", 1 << cpu, irq);
 
         LCONSOLE_INFO("Binding irq %u to CPU %d with cmd: %s\n",
-		      irq, cpu, cmdline);
+                      irq, cpu, cmdline);
 
         /* FIXME: Find a better method of setting IRQ affinity...
          */
@@ -162,8 +275,8 @@ int
 ksocknal_lib_get_conn_addrs (ksock_conn_t *conn)
 {
         int rc = libcfs_sock_getaddr(conn->ksnc_sock, 1,
-				     &conn->ksnc_ipaddr,
-				     &conn->ksnc_port);
+                                     &conn->ksnc_ipaddr,
+                                     &conn->ksnc_port);
 
         /* Didn't need the {get,put}connsock dance to deref ksnc_sock... */
         LASSERT (!conn->ksnc_closing);
@@ -174,7 +287,7 @@ ksocknal_lib_get_conn_addrs (ksock_conn_t *conn)
         }
 
         rc = libcfs_sock_getaddr(conn->ksnc_sock, 0,
-				 &conn->ksnc_myipaddr, NULL);
+                                 &conn->ksnc_myipaddr, NULL);
         if (rc != 0) {
                 CERROR ("Error %d getting sock local IP\n", rc);
                 return rc;
@@ -187,7 +300,7 @@ unsigned int
 ksocknal_lib_sock_irq (struct socket *sock)
 {
         int                irq = 0;
-#if CPU_AFFINITY
+#ifdef CPU_AFFINITY
         struct dst_entry  *dst;
 
         if (!*ksocknal_tunables.ksnd_irq_affinity)
@@ -213,7 +326,7 @@ int
 ksocknal_lib_zc_capable(struct socket *sock)
 {
         int  caps = sock->sk->sk_route_caps;
-        
+
         /* ZC if the socket supports scatter/gather and doesn't need software
          * checksums */
         return ((caps & NETIF_F_SG) != 0 &&
@@ -270,7 +383,7 @@ ksocknal_lib_send_iov (ksock_conn_t *conn, ksock_tx_t *tx)
                 rc = sock_sendmsg(sock, &msg, nob);
                 set_fs (oldmm);
         }
-	return rc;
+        return rc;
 }
 
 int
@@ -287,6 +400,7 @@ ksocknal_lib_send_kiov (ksock_conn_t *conn, ksock_tx_t *tx)
         if (kiov->kiov_len >= *ksocknal_tunables.ksnd_zc_min_frag &&
             tx->tx_msg.ksm_zc_req_cookie != 0) {
                 /* Zero copy is enabled */
+                struct sock   *sk = sock->sk;
                 struct page   *page = kiov->kiov_page;
                 int            offset = kiov->kiov_offset;
                 int            fragsize = kiov->kiov_len;
@@ -299,7 +413,12 @@ ksocknal_lib_send_kiov (ksock_conn_t *conn, ksock_tx_t *tx)
                     fragsize < tx->tx_resid)
                         msgflg |= MSG_MORE;
 
-                rc = tcp_sendpage(sock, page, offset, fragsize, msgflg);
+                if (sk->sk_prot->sendpage != NULL) {
+                        rc = sk->sk_prot->sendpage(sk, page,
+                                                   offset, fragsize, msgflg);
+                } else {
+                        rc = tcp_sendpage(sock, page, offset, fragsize, msgflg);
+                }
         } else {
 #if SOCKNAL_SINGLE_FRAG_TX || !SOCKNAL_RISK_KMAP_DEADLOCK
                 struct iovec  scratch;
@@ -341,7 +460,7 @@ ksocknal_lib_send_kiov (ksock_conn_t *conn, ksock_tx_t *tx)
                 for (i = 0; i < niov; i++)
                         kunmap(kiov[i].kiov_page);
         }
-	return rc;
+        return rc;
 }
 
 void
@@ -420,14 +539,14 @@ ksocknal_lib_recv_iov (ksock_conn_t *conn)
                         fragnob = iov[i].iov_len;
                         if (fragnob > sum)
                                 fragnob = sum;
-                
-                        conn->ksnc_rx_csum = ksocknal_csum(conn->ksnc_rx_csum, 
+
+                        conn->ksnc_rx_csum = ksocknal_csum(conn->ksnc_rx_csum,
                                                            iov[i].iov_base, fragnob);
                 }
                 conn->ksnc_msg.ksm_csum = saved_csum;
         }
 
-	return rc;
+        return rc;
 }
 
 int
@@ -487,7 +606,7 @@ ksocknal_lib_recv_kiov (ksock_conn_t *conn)
                         fragnob = kiov[i].kiov_len;
                         if (fragnob > sum)
                                 fragnob = sum;
-                
+
                         conn->ksnc_rx_csum = ksocknal_csum(conn->ksnc_rx_csum,
                                                            base, fragnob);
 
@@ -497,10 +616,11 @@ ksocknal_lib_recv_kiov (ksock_conn_t *conn)
         for (i = 0; i < niov; i++)
                 kunmap(kiov[i].kiov_page);
 
-	return (rc);
+        return (rc);
 }
 
-void ksocknal_lib_csum_tx(ksock_tx_t *tx)
+void
+ksocknal_lib_csum_tx(ksock_tx_t *tx)
 {
         int          i;
         __u32        csum;
@@ -553,13 +673,13 @@ ksocknal_lib_get_conn_tunables (ksock_conn_t *conn, int *txmem, int *rxmem, int
                 return (-ESHUTDOWN);
         }
 
-	rc = libcfs_sock_getbuf(sock, txmem, rxmem);
+        rc = libcfs_sock_getbuf(sock, txmem, rxmem);
         if (rc == 0) {
                 len = sizeof(*nagle);
-		set_fs(KERNEL_DS);
+                set_fs(KERNEL_DS);
                 rc = sock->ops->getsockopt(sock, SOL_TCP, TCP_NODELAY,
                                            (char *)nagle, &len);
-		set_fs(oldmm);
+                set_fs(oldmm);
         }
 
         ksocknal_connsock_decref(conn);
@@ -624,15 +744,15 @@ ksocknal_lib_setup_sock (struct socket *sock)
                 }
         }
 
-	rc = libcfs_sock_setbuf(sock,
+        rc = libcfs_sock_setbuf(sock,
                                 *ksocknal_tunables.ksnd_tx_buffer_size,
                                 *ksocknal_tunables.ksnd_rx_buffer_size);
-	if (rc != 0) {
-		CERROR ("Can't set buffer tx %d, rx %d buffers: %d\n",
+        if (rc != 0) {
+                CERROR ("Can't set buffer tx %d, rx %d buffers: %d\n",
                         *ksocknal_tunables.ksnd_tx_buffer_size,
                         *ksocknal_tunables.ksnd_rx_buffer_size, rc);
-		return (rc);
-	}
+                return (rc);
+        }
 
 /* TCP_BACKOFF_* sockopt tunables unsupported in stock kernels */
 #ifdef SOCKNAL_BACKOFF
@@ -792,7 +912,7 @@ ksocknal_data_ready (struct sock *sk, int n)
                 LASSERT (sk->sk_data_ready != &ksocknal_data_ready);
                 sk->sk_data_ready (sk, n);
         } else
-		ksocknal_read_callback(conn);
+                ksocknal_read_callback(conn);
 
         read_unlock (&ksocknal_data.ksnd_global_lock);
 
@@ -832,11 +952,11 @@ ksocknal_write_space (struct sock *sk)
         }
 
         if (wspace >= min_wpace) {              /* got enough space */
-		ksocknal_write_callback(conn);
+                ksocknal_write_callback(conn);
 
-		/* Clear SOCK_NOSPACE _after_ ksocknal_write_callback so the
-		 * ENOMEM check in ksocknal_transmit is race-free (think about
-		 * it). */
+                /* Clear SOCK_NOSPACE _after_ ksocknal_write_callback so the
+                 * ENOMEM check in ksocknal_transmit is race-free (think about
+                 * it). */
 
                 clear_bit (SOCK_NOSPACE, &sk->sk_socket->flags);
         }
@@ -847,33 +967,33 @@ ksocknal_write_space (struct sock *sk)
 void
 ksocknal_lib_save_callback(struct socket *sock, ksock_conn_t *conn)
 {
-	conn->ksnc_saved_data_ready = sock->sk->sk_data_ready;
-	conn->ksnc_saved_write_space = sock->sk->sk_write_space;
+        conn->ksnc_saved_data_ready = sock->sk->sk_data_ready;
+        conn->ksnc_saved_write_space = sock->sk->sk_write_space;
 }
 
 void
 ksocknal_lib_set_callback(struct socket *sock,  ksock_conn_t *conn)
 {
-	sock->sk->sk_user_data = conn;
-	sock->sk->sk_data_ready = ksocknal_data_ready;
-	sock->sk->sk_write_space = ksocknal_write_space;
-	return;
+        sock->sk->sk_user_data = conn;
+        sock->sk->sk_data_ready = ksocknal_data_ready;
+        sock->sk->sk_write_space = ksocknal_write_space;
+        return;
 }
 
 void
 ksocknal_lib_reset_callback(struct socket *sock, ksock_conn_t *conn)
 {
-	/* Remove conn's network callbacks.
-	 * NB I _have_ to restore the callback, rather than storing a noop,
-	 * since the socket could survive past this module being unloaded!! */
-	sock->sk->sk_data_ready = conn->ksnc_saved_data_ready;
-	sock->sk->sk_write_space = conn->ksnc_saved_write_space;
+        /* Remove conn's network callbacks.
+         * NB I _have_ to restore the callback, rather than storing a noop,
+         * since the socket could survive past this module being unloaded!! */
+        sock->sk->sk_data_ready = conn->ksnc_saved_data_ready;
+        sock->sk->sk_write_space = conn->ksnc_saved_write_space;
 
-	/* A callback could be in progress already; they hold a read lock
-	 * on ksnd_global_lock (to serialise with me) and NOOP if
-	 * sk_user_data is NULL. */
-	sock->sk->sk_user_data = NULL;
+        /* A callback could be in progress already; they hold a read lock
+         * on ksnd_global_lock (to serialise with me) and NOOP if
+         * sk_user_data is NULL. */
+        sock->sk->sk_user_data = NULL;
 
-	return ;
+        return ;
 }
 
