@@ -120,13 +120,13 @@ void cfs_waitq_signal_nr(struct cfs_waitq *waitq, int nr)
         (void)waitq;
 }
 
-void cfs_waitq_broadcast(struct cfs_waitq *waitq, int state)
+void cfs_waitq_broadcast(struct cfs_waitq *waitq)
 {
         LASSERT(waitq != NULL);
         (void)waitq;
 }
 
-void cfs_waitq_wait(struct cfs_waitlink *link)
+void cfs_waitq_wait(struct cfs_waitlink *link, int state)
 {
         LASSERT(link != NULL);
         (void)link;
@@ -276,13 +276,6 @@ void cfs_mem_cache_free(cfs_mem_cache_t *c, void *addr)
         cfs_free(addr);
 }
 
-/*
- * This uses user-visible declarations from <linux/kdev_t.h>
- */
-#ifdef __LINUX__
-#include <linux/kdev_t.h>
-#endif
-
 void cfs_enter_debugger(void)
 {
         /*
@@ -345,7 +338,7 @@ void cfs_clear_sigpending(void)
         return;
 }
 
-#ifdef __LINUX__
+#ifdef __linux__
 
 /*
  * In glibc (NOT in Linux, so check above is not right), implement
@@ -375,7 +368,7 @@ void *cfs_stack_trace_frame(struct cfs_stack_trace *trace, int frame_no)
         return NULL;
 }
 
-/* __LINUX__ */
+/* __linux__ */
 #endif
 
 void lbug_with_loc(char *file, const char *func, const int line)
@@ -385,7 +378,6 @@ void lbug_with_loc(char *file, const char *func, const int line)
         abort();
 }
 
-
 /* !__KERNEL__ */
 #endif
 
