@@ -144,7 +144,8 @@ int llu_local_open(struct llu_inode_info *lli, struct lookup_intent *it)
 
         body = lustre_msg_buf(req->rq_repmsg, DLM_REPLY_REC_OFF, sizeof(*body));
         LASSERT(body != NULL);                 /* reply already checked out */
-        LASSERT_REPSWABBED(req, DLM_REPLY_REC_OFF);       /* and swabbed down */
+        /* and swabbed down */
+        LASSERT(lustre_rep_swabbed(req, DLM_REPLY_REC_OFF));
 
         /* already opened? */
         if (lli->lli_open_count++)
@@ -435,7 +436,7 @@ int llu_iop_close(struct inode *inode)
         }
         /* if open count == 0 && stale_flag is set, should we
          * remove the inode immediately? */
-        liblustre_wait_event(0);
+        liblustre_wait_idle();
         return 0;
 }
 
