@@ -73,10 +73,13 @@ static inline unsigned ll_dir_rec_len(unsigned name_len)
         return (name_len + 8 + LL_DIR_PAD - 1) & ~(LL_DIR_PAD - 1);
 }
 
-
+#ifndef HAVE_PAGE_CHECKED
 #ifdef HAVE_PG_FS_MISC
 #define PageChecked(page)        test_bit(PG_fs_misc, &(page)->flags)
 #define SetPageChecked(page)     set_bit(PG_fs_misc, &(page)->flags)
+#else
+#error PageChecked or PageFsMisc not defined in kernel
+#endif
 #endif
 
 /* returns the page unlocked, but with a reference */
@@ -103,7 +106,7 @@ static int ll_dir_readpage(struct file *file, struct page *page)
                                       sizeof(*body));
                 LASSERT(body != NULL); /* checked by mdc_readpage() */
                 /* swabbed by mdc_readpage() */
-                LASSERT_REPSWABBED(request, REPLY_REC_OFF);
+                LASSERT(lustre_rep_swabbed(request, REPLY_REC_OFF));
 
                 if (body->size != i_size_read(inode)) {
                         ll_inode_size_lock(inode, 0);
@@ -594,7 +597,7 @@ int ll_dir_getstripe(struct inode *inode, struct lov_mds_md **lmmp,
                         sizeof(*body));
         LASSERT(body != NULL); /* checked by mdc_getattr_name */
         /* swabbed by mdc_getattr_name */
-        LASSERT_REPSWABBED(req, REPLY_REC_OFF);
+        LASSERT(lustre_rep_swabbed(req, REPLY_REC_OFF));
 
         lmmsize = body->eadatasize;
 
@@ -605,7 +608,7 @@ int ll_dir_getstripe(struct inode *inode, struct lov_mds_md **lmmp,
 
         lmm = lustre_msg_buf(req->rq_repmsg, REPLY_REC_OFF + 1, lmmsize);
         LASSERT(lmm != NULL);
-        LASSERT_REPSWABBED(req, REPLY_REC_OFF + 1);
+        LASSERT(lustre_rep_swabbed(req, REPLY_REC_OFF + 1));
 
         /*
          * This is coming from the MDS, so is probably in
@@ -614,7 +617,6 @@ int ll_dir_getstripe(struct inode *inode, struct lov_mds_md **lmmp,
          */
         if (lmm->lmm_magic == __swab32(LOV_MAGIC)) {
                 lustre_swab_lov_user_md((struct lov_user_md *)lmm);
-                lustre_swab_lov_user_md_objects((struct lov_user_md *)lmm);
         }
 out:
         *lmmp = lmm;
@@ -735,7 +737,7 @@ static int ll_dir_ioctl(struct inode *inode, struct file *file,
                                               sizeof(*body));
                         LASSERT(body != NULL); /* checked by mdc_getattr_name */
                         /* swabbed by mdc_getattr_name */
-                        LASSERT_REPSWABBED(request, REPLY_REC_OFF);
+                        LASSERT(lustre_rep_swabbed(request, REPLY_REC_OFF));
                 } else {
                         GOTO(out_req, rc);
                 }
@@ -967,6 +969,7 @@ static int ll_dir_ioctl(struct inode *inode, struct file *file,
                 type = qctl->qc_type;
                 id = qctl->qc_id;
                 switch (cmd) {
+                case LUSTRE_Q_INVALIDATE:
                 case Q_QUOTAON:
                 case Q_QUOTAOFF:
                 case Q_SETQUOTA:
@@ -982,9 +985,12 @@ static int ll_dir_ioctl(struct inode *inode, struct file *file,
 
                         /* XXX: dqb_valid is borrowed as a flag to mark that
                          *      only mds quota is wanted */
-                        if (qctl->qc_dqblk.dqb_valid)
+                        if (qctl->qc_dqblk.dqb_valid) {
                                 qctl->obd_uuid = sbi->ll_mdc_exp->exp_obd->
                                                         u.cli.cl_target_uuid;
+                                qctl->qc_dqblk.dqb_valid = 0;
+                        }
+
                         break;
                 case Q_GETINFO:
                         break;
