@@ -60,8 +60,8 @@ static int lov_wr_stripesize(struct file *file, const char *buffer,
         if (rc)
                 return rc;
 
+        lov_fix_desc_stripe_size(&val);
         desc->ld_default_stripe_size = val;
-        lov_fix_desc(desc);
         return count;
 }
 
@@ -92,7 +92,6 @@ static int lov_wr_stripeoffset(struct file *file, const char *buffer,
                 return rc;
 
         desc->ld_default_stripe_offset = val;
-        lov_fix_desc(desc);
         return count;
 }
 
@@ -121,8 +120,8 @@ static int lov_wr_stripetype(struct file *file, const char *buffer,
         if (rc)
                 return rc;
 
+        lov_fix_desc_pattern(&val);
         desc->ld_pattern = val;
-        lov_fix_desc(desc);
         return count;
 }
 
@@ -135,7 +134,8 @@ static int lov_rd_stripecount(char *page, char **start, off_t off, int count,
         LASSERT(dev != NULL);
         desc = &dev->u.lov.desc;
         *eof = 1;
-        return snprintf(page, count, "%d\n", desc->ld_default_stripe_count);
+        return snprintf(page, count, "%ld\n",
+                        (long)(desc->ld_default_stripe_count + 1) - 1);
 }
 
 static int lov_wr_stripecount(struct file *file, const char *buffer,
@@ -151,8 +151,8 @@ static int lov_wr_stripecount(struct file *file, const char *buffer,
         if (rc)
                 return rc;
 
+        lov_fix_desc_stripe_count(&val);
         desc->ld_default_stripe_count = val;
-        lov_fix_desc(desc);
         return count;
 }
 
@@ -264,8 +264,12 @@ static void *lov_tgt_seq_start(struct seq_file *p, loff_t *pos)
         struct obd_device *dev = p->private;
         struct lov_obd *lov = &dev->u.lov;
 
-        return (*pos >= lov->desc.ld_tgt_count) ? NULL : lov->lov_tgts[*pos];
-
+        while (*pos < lov->desc.ld_tgt_count) {
+                if (lov->lov_tgts[*pos])
+                        return lov->lov_tgts[*pos];
+                ++*pos;
+        }
+        return NULL;
 }
 
 static void lov_tgt_seq_stop(struct seq_file *p, void *v)
@@ -317,7 +321,7 @@ static int lov_target_seq_open(struct inode *inode, struct file *file)
         return 0;
 }
 
-struct lprocfs_vars lprocfs_obd_vars[] = {
+struct lprocfs_vars lprocfs_lov_obd_vars[] = {
         { "uuid",         lprocfs_rd_uuid,        0, 0 },
         { "stripesize",   lov_rd_stripesize,      lov_wr_stripesize, 0 },
         { "stripeoffset", lov_rd_stripeoffset,    lov_wr_stripeoffset, 0 },
@@ -338,11 +342,17 @@ struct lprocfs_vars lprocfs_obd_vars[] = {
         { 0 }
 };
 
-static struct lprocfs_vars lprocfs_module_vars[] = {
+static struct lprocfs_vars lprocfs_lov_module_vars[] = {
         { "num_refs",     lprocfs_rd_numrefs,     0, 0 },
         { 0 }
 };
 
+void lprocfs_lov_init_vars(struct lprocfs_static_vars *lvars)
+{
+    lvars->module_vars  = lprocfs_lov_module_vars;
+    lvars->obd_vars     = lprocfs_lov_obd_vars;
+}
+
 struct file_operations lov_proc_target_fops = {
         .owner   = THIS_MODULE,
         .open    = lov_target_seq_open,
@@ -350,6 +360,5 @@ struct file_operations lov_proc_target_fops = {
         .llseek  = seq_lseek,
         .release = lprocfs_seq_release,
 };
-
-LPROCFS_INIT_VARS(lov, lprocfs_module_vars, lprocfs_obd_vars)
 #endif /* LPROCFS */
+
