@@ -87,6 +87,14 @@ int mdc_resource_get_unused(struct obd_export *exp, struct ll_fid *fid,
         RETURN(count);
 }
 
+struct ptlrpc_request *mdc_prep_elc_req(struct obd_export *exp,
+                                        int bufcount, int *size, int off,
+                                        struct list_head *cancels, int count)
+{
+        return ldlm_prep_elc_req(exp, LUSTRE_MDS_VERSION, MDS_REINT,
+                                 bufcount, size, off, 0, cancels, count);
+}
+
 /* If mdc_setattr is called with an 'iattr', then it is a normal RPC that
  * should take the normal semaphore and go to the normal portal.
  *
@@ -103,7 +111,8 @@ int mdc_setattr(struct obd_export *exp, struct mdc_op_data *op_data,
         struct mdc_rpc_lock *rpc_lock;
         struct obd_device *obd = exp->exp_obd;
         int size[5] = { sizeof(struct ptlrpc_body),
-                        sizeof(*rec), ealen, ea2len, 0 };
+                        sizeof(*rec), ealen, ea2len,
+                        sizeof(struct ldlm_request) };
         int count, bufcount = 2, rc;
         __u64 bits;
         ENTRY;
@@ -121,22 +130,16 @@ int mdc_setattr(struct obd_export *exp, struct mdc_op_data *op_data,
                 bits |= MDS_INODELOCK_LOOKUP;
         count = mdc_resource_get_unused(exp, &op_data->fid1,
                                         &cancels, LCK_EX, bits);
-        if (exp_connect_cancelset(exp) && count) {
+        if (exp_connect_cancelset(exp))
                 bufcount = 5;
-                size[REQ_REC_OFF + 3] = ldlm_request_bufsize(count, MDS_REINT);
-        }
-        req = ptlrpc_prep_req(class_exp2cliimp(exp), LUSTRE_MDS_VERSION,
-                              MDS_REINT, bufcount, size, NULL);
-        if (exp_connect_cancelset(exp) && req)
-                ldlm_cli_cancel_list(&cancels, count, req, REQ_REC_OFF + 3);
-        else
-                ldlm_lock_list_put(&cancels, l_bl_ast, count);
-
+        req = mdc_prep_elc_req(exp, bufcount, size,
+                               REQ_REC_OFF + 3, &cancels, count);
         if (req == NULL)
                 RETURN(-ENOMEM);
 
         if (iattr->ia_valid & ATTR_FROM_OPEN) {
-                req->rq_request_portal = MDS_SETATTR_PORTAL; //XXX FIXME bug 249
+                req->rq_request_portal = MDS_SETATTR_PORTAL;
+                ptlrpc_at_set_req_timeout(req);
                 rpc_lock = obd->u.cli.cl_setattr_lock;
         } else {
                 rpc_lock = obd->u.cli.cl_rpc_lock;
@@ -169,7 +172,7 @@ int mdc_create(struct obd_export *exp, struct mdc_op_data *op_data,
         int level, bufcount = 3, rc;
         int size[5] = { sizeof(struct ptlrpc_body),
                         sizeof(struct mds_rec_create),
-                        op_data->namelen + 1, 0, 0 };
+                        op_data->namelen + 1, 0, sizeof(struct ldlm_request) };
         int count;
         ENTRY;
 
@@ -180,17 +183,10 @@ int mdc_create(struct obd_export *exp, struct mdc_op_data *op_data,
 
         count = mdc_resource_get_unused(exp, &op_data->fid1, &cancels,
                                         LCK_EX, MDS_INODELOCK_UPDATE);
-        if (exp_connect_cancelset(exp) && count) {
+        if (exp_connect_cancelset(exp))
                 bufcount = 5;
-                size[REQ_REC_OFF + 3] = ldlm_request_bufsize(count, MDS_REINT);
-        }
-        req = ptlrpc_prep_req(class_exp2cliimp(exp), LUSTRE_MDS_VERSION,
-                              MDS_REINT, bufcount, size, NULL);
-        if (exp_connect_cancelset(exp) && req)
-                ldlm_cli_cancel_list(&cancels, count, req, REQ_REC_OFF + 3);
-        else
-                ldlm_lock_list_put(&cancels, l_bl_ast, count);
-
+        req = mdc_prep_elc_req(exp, bufcount, size,
+                               REQ_REC_OFF + 3, &cancels, count);
         if (req == NULL)
                 RETURN(-ENOMEM);
 
@@ -226,7 +222,7 @@ int mdc_unlink(struct obd_export *exp, struct mdc_op_data *op_data,
         struct ptlrpc_request *req = *request;
         int size[4] = { sizeof(struct ptlrpc_body),
                         sizeof(struct mds_rec_unlink),
-                        op_data->namelen + 1, 0 };
+                        op_data->namelen + 1, sizeof(struct ldlm_request) };
         int count, rc, bufcount = 3;
         ENTRY;
 
@@ -236,17 +232,10 @@ int mdc_unlink(struct obd_export *exp, struct mdc_op_data *op_data,
         if (op_data->fid3.id)
                 count += mdc_resource_get_unused(exp, &op_data->fid3, &cancels,
                                                  LCK_EX, MDS_INODELOCK_FULL);
-        if (exp_connect_cancelset(exp) && count) {
+        if (exp_connect_cancelset(exp))
                 bufcount = 4;
-                size[REQ_REC_OFF + 2] = ldlm_request_bufsize(count, MDS_REINT);
-        }
-        req = ptlrpc_prep_req(class_exp2cliimp(exp), LUSTRE_MDS_VERSION,
-                              MDS_REINT, bufcount, size, NULL);
-        if (exp_connect_cancelset(exp) && req)
-                ldlm_cli_cancel_list(&cancels, count, req, REQ_REC_OFF + 2);
-        else
-                ldlm_lock_list_put(&cancels, l_bl_ast, count);
-
+        req = mdc_prep_elc_req(exp, bufcount, size,
+                               REQ_REC_OFF + 2, &cancels, count);
         if (req == NULL)
                 RETURN(-ENOMEM);
         *request = req;
@@ -272,7 +261,7 @@ int mdc_link(struct obd_export *exp, struct mdc_op_data *op_data,
         struct ptlrpc_request *req;
         int size[4] = { sizeof(struct ptlrpc_body),
                         sizeof(struct mds_rec_link),
-                        op_data->namelen + 1, 0 };
+                        op_data->namelen + 1, sizeof(struct ldlm_request) };
         int count, rc, bufcount = 3;
         ENTRY;
 
@@ -280,17 +269,10 @@ int mdc_link(struct obd_export *exp, struct mdc_op_data *op_data,
                                         LCK_EX, MDS_INODELOCK_UPDATE);
         count += mdc_resource_get_unused(exp, &op_data->fid2, &cancels,
                                          LCK_EX, MDS_INODELOCK_UPDATE);
-        if (exp_connect_cancelset(exp) && count) {
+        if (exp_connect_cancelset(exp))
                 bufcount = 4;
-                size[REQ_REC_OFF + 2] = ldlm_request_bufsize(count, MDS_REINT);
-        }
-        req = ptlrpc_prep_req(class_exp2cliimp(exp), LUSTRE_MDS_VERSION,
-                              MDS_REINT, bufcount, size, NULL);
-        if (exp_connect_cancelset(exp) && req)
-                ldlm_cli_cancel_list(&cancels, count, req, REQ_REC_OFF + 2);
-        else
-                ldlm_lock_list_put(&cancels, l_bl_ast, count);
-
+        req = mdc_prep_elc_req(exp, bufcount, size,
+                               REQ_REC_OFF + 2, &cancels, count);
         if (req == NULL)
                 RETURN(-ENOMEM);
 
@@ -316,7 +298,7 @@ int mdc_rename(struct obd_export *exp, struct mdc_op_data *op_data,
         struct ptlrpc_request *req;
         int size[5] = { sizeof(struct ptlrpc_body),
                         sizeof(struct mds_rec_rename),
-                        oldlen + 1, newlen + 1, 0 };
+                        oldlen + 1, newlen + 1, sizeof(struct ldlm_request) };
         int count, rc, bufcount = 4;
         ENTRY;
 
@@ -330,17 +312,10 @@ int mdc_rename(struct obd_export *exp, struct mdc_op_data *op_data,
         if (op_data->fid4.id)
                 count += mdc_resource_get_unused(exp, &op_data->fid4, &cancels,
                                                  LCK_EX, MDS_INODELOCK_FULL);
-        if (exp_connect_cancelset(exp) && count) {
+        if (exp_connect_cancelset(exp))
                 bufcount = 5;
-                size[REQ_REC_OFF + 3] = ldlm_request_bufsize(count, MDS_REINT);
-        }
-        req = ptlrpc_prep_req(class_exp2cliimp(exp), LUSTRE_MDS_VERSION,
-                              MDS_REINT, bufcount, size, NULL);
-        if (exp_connect_cancelset(exp) && req)
-                ldlm_cli_cancel_list(&cancels, count, req, REQ_REC_OFF + 3);
-        else
-                ldlm_lock_list_put(&cancels, l_bl_ast, count);
-
+        req = mdc_prep_elc_req(exp, bufcount, size,
+                               REQ_REC_OFF + 3, &cancels, count);
         if (req == NULL)
                 RETURN(-ENOMEM);
 
