@@ -144,7 +144,7 @@ int mdc_getattr_common(struct obd_export *exp, unsigned int ea_size,
 
         CDEBUG(D_NET, "mode: %o\n", body->mode);
 
-        LASSERT_REPSWAB(req, REPLY_REC_OFF + 1);
+        lustre_set_rep_swabbed(req, REPLY_REC_OFF + 1);
         if (body->eadatasize != 0) {
                 /* reply indicates presence of eadata; check it's there... */
                 eadata = lustre_msg_buf(req->rq_repmsg, REPLY_REC_OFF + 1,
@@ -419,7 +419,7 @@ int mdc_req2lustre_md(struct ptlrpc_request *req, int offset,
 
         md->body = lustre_msg_buf(req->rq_repmsg, offset, sizeof (*md->body));
         LASSERT (md->body != NULL);
-        LASSERT_REPSWABBED(req, offset);
+        LASSERT(lustre_rep_swabbed(req, offset));
         offset++;
 
         if (md->body->valid & OBD_MD_FLEASIZE) {
@@ -442,7 +442,7 @@ int mdc_req2lustre_md(struct ptlrpc_request *req, int offset,
                         CERROR ("incorrect message: lmm == 0\n");
                         GOTO(err_out, rc = -EPROTO);
                 }
-                LASSERT_REPSWABBED(req, offset);
+                LASSERT(lustre_rep_swabbed(req, offset));
 
                 rc = obd_unpackmd(exp, &md->lsm, lmm, lmmsize);
                 if (rc < 0)
@@ -450,7 +450,7 @@ int mdc_req2lustre_md(struct ptlrpc_request *req, int offset,
 
                 if (rc < sizeof(*md->lsm)) {
                         CERROR ("lsm size too small:  rc < sizeof (*md->lsm) "
-                                "(%d < "LPSZ")\n", rc, sizeof(*md->lsm));
+                                "(%d < %d)\n", rc, (int)sizeof(*md->lsm));
                         GOTO(err_out, rc = -EPROTO);
                 }
                 rc = 0;
@@ -573,9 +573,13 @@ void mdc_set_open_replay_data(struct obd_client_handle *och,
                                                DLM_REPLY_REC_OFF,
                                                sizeof(*body));
 
+        /* If request is not eligible for replay, just bail out */
+        if (!open_req->rq_replay)
+                return;
+
         /* incoming message in my byte order (it's been swabbed) */
         LASSERT(rec != NULL);
-        LASSERT_REPSWABBED(open_req, DLM_REPLY_REC_OFF);
+        LASSERT(lustre_rep_swabbed(open_req, DLM_REPLY_REC_OFF));
         /* outgoing messages always in my byte order */
         LASSERT(body != NULL);
 
@@ -587,12 +591,6 @@ void mdc_set_open_replay_data(struct obd_client_handle *och,
                 }
 
                 spin_lock(&open_req->rq_lock);
-                if (!open_req->rq_replay) {
-                        OBD_FREE(mod, sizeof(*mod));
-                        spin_unlock(&open_req->rq_lock);
-                        return;
-                }
-
                 och->och_mod = mod;
                 mod->mod_och = och;
                 mod->mod_open_req = open_req;
@@ -678,8 +676,8 @@ int mdc_close(struct obd_export *exp, struct obdo *oa,
         /* To avoid a livelock (bug 7034), we need to send CLOSE RPCs to a
          * portal whose threads are not taking any DLM locks and are therefore
          * always progressing */
-        /* XXX FIXME bug 249 */
         req->rq_request_portal = MDS_READPAGE_PORTAL;
+        ptlrpc_at_set_req_timeout(req);
 
         /* Ensure that this close's handle is fixed up during replay. */
         LASSERT(och != NULL);
@@ -787,8 +785,8 @@ int mdc_readpage(struct obd_export *exp, struct ll_fid *fid, __u64 offset,
         if (req == NULL)
                 GOTO(out, rc = -ENOMEM);
 
-        /* XXX FIXME bug 249 */
         req->rq_request_portal = MDS_READPAGE_PORTAL;
+        ptlrpc_at_set_req_timeout(req);
 
         desc = ptlrpc_prep_bulk_imp(req, 1, BULK_PUT_SINK, MDS_BULK_PORTAL);
         if (desc == NULL)
@@ -980,7 +978,7 @@ int mdc_get_info(struct obd_export *exp, __u32 keylen, void *key,
 }
 
 static int mdc_statfs(struct obd_device *obd, struct obd_statfs *osfs,
-                      __u64 max_age)
+                      __u64 max_age, __u32 flags)
 {
         struct ptlrpc_request *req;
         struct obd_statfs *msfs;
@@ -1000,6 +998,12 @@ static int mdc_statfs(struct obd_device *obd, struct obd_statfs *osfs,
 
         ptlrpc_req_set_repsize(req, 2, size);
 
+        if (flags & OBD_STATFS_NODELAY) {
+                /* procfs requests not want stay in wait for avoid deadlock */
+                req->rq_no_resend = 1;
+                req->rq_no_delay = 1;
+        }
+
         rc = ptlrpc_queue_wait(req);
 
         if (rc)
@@ -1169,7 +1173,7 @@ static int mdc_import_event(struct obd_device *obd, struct obd_import *imp,
 static int mdc_setup(struct obd_device *obd, obd_count len, void *buf)
 {
         struct client_obd *cli = &obd->u.cli;
-        struct lprocfs_static_vars lvars;
+        struct lprocfs_static_vars lvars = { 0 };
         int rc;
         ENTRY;
 
@@ -1193,8 +1197,9 @@ static int mdc_setup(struct obd_device *obd, obd_count len, void *buf)
         rc = client_obd_setup(obd, len, buf);
         if (rc)
                 GOTO(err_close_lock, rc);
-        lprocfs_init_vars(mdc, &lvars);
-        lprocfs_obd_setup(obd, lvars.obd_vars);
+        lprocfs_mdc_init_vars(&lvars);
+        if (lprocfs_obd_setup(obd, lvars.obd_vars) == 0)
+                ptlrpc_lprocfs_register_obd(obd);
 
         rc = obd_llog_init(obd, obd, 0, NULL, NULL);
         if (rc) {
@@ -1295,6 +1300,7 @@ static int mdc_cleanup(struct obd_device *obd)
         OBD_FREE(cli->cl_setattr_lock, sizeof (*cli->cl_setattr_lock));
         OBD_FREE(cli->cl_close_lock, sizeof (*cli->cl_close_lock));
 
+        ptlrpc_lprocfs_unregister_obd(obd);
         lprocfs_obd_cleanup(obd);
         ptlrpcd_decref();
 
@@ -1345,10 +1351,10 @@ static int mdc_llog_finish(struct obd_device *obd, int count)
 static int mdc_process_config(struct obd_device *obd, obd_count len, void *buf)
 {
         struct lustre_cfg *lcfg = buf;
-        struct lprocfs_static_vars lvars;
+        struct lprocfs_static_vars lvars = { 0 };
         int rc = 0;
 
-        lprocfs_init_vars(mdc, &lvars);
+        lprocfs_mdc_init_vars(&lvars);
         
         rc = class_process_proc_param(PARAM_MDC, lvars.obd_vars, lcfg, obd);
         return(rc);
@@ -1378,8 +1384,8 @@ struct obd_ops mdc_obd_ops = {
 int __init mdc_init(void)
 {
         int rc;
-        struct lprocfs_static_vars lvars;
-        lprocfs_init_vars(mdc, &lvars);
+        struct lprocfs_static_vars lvars = { 0 };
+        lprocfs_mdc_init_vars(&lvars);
         request_module("lquota");
         quota_interface = PORTAL_SYMBOL_GET(mdc_quota_interface);
         init_obd_quota_ops(quota_interface, &mdc_obd_ops);
