@@ -174,10 +174,14 @@ int mds_finish_transno(struct mds_obd *mds, struct inode *inode, void *handle,
                 mcd->mcd_last_close_data = cpu_to_le32(op_data);
         } else {
                 prev_transno = le64_to_cpu(mcd->mcd_last_transno);
-                mcd->mcd_last_transno = cpu_to_le64(transno);
-                mcd->mcd_last_xid = cpu_to_le64(req->rq_xid);
-                mcd->mcd_last_result = cpu_to_le32(rc);
-                mcd->mcd_last_data = cpu_to_le32(op_data);
+                if (((lustre_msg_get_flags(req->rq_reqmsg) &
+                      (MSG_RESENT | MSG_REPLAY)) == 0) ||
+                    (transno > prev_transno)) {
+                        mcd->mcd_last_transno = cpu_to_le64(transno);
+                        mcd->mcd_last_xid     = cpu_to_le64(req->rq_xid);
+                        mcd->mcd_last_result  = cpu_to_le32(rc);
+                        mcd->mcd_last_data    = cpu_to_le32(op_data);
+                }
         }
         /* update the server data to not lose the greatest transno. Bug 11125 */
         if ((transno == 0) && (prev_transno == mds->mds_last_transno))
@@ -574,7 +578,7 @@ static int mds_reint_setattr(struct mds_update_record *rec, int offset,
                         GOTO(cleanup, rc = -ENOMEM);
 
                 cleanup_phase = 2;
-                rc = mds_get_md(obd, inode, lmm, &lmm_size, need_lock);
+                rc = mds_get_md(obd, inode, lmm, &lmm_size, need_lock, 0);
                 if (rc < 0)
                         GOTO(cleanup, rc);
                 rc = 0;
@@ -772,6 +776,8 @@ static int mds_reint_create(struct mds_update_record *rec, int offset,
         unsigned int qcids[MAXQUOTAS] = { current->fsuid, current->fsgid };
         unsigned int qpids[MAXQUOTAS] = { 0, 0 };
         struct lvfs_dentry_params dp = LVFS_DENTRY_PARAMS_INIT;
+        int rec_pending = 0;
+        unsigned int gid = current->fsgid;
         ENTRY;
 
         LASSERT(offset == REQ_REC_OFF);
@@ -831,6 +837,16 @@ static int mds_reint_create(struct mds_update_record *rec, int offset,
         dp.ldp_inum = (unsigned long)rec->ur_fid2->id;
         dp.ldp_ptr = req;
 
+        if (dir->i_mode & S_ISGID)
+                gid = dir->i_gid;
+        else
+                gid = current->fsgid;
+
+        /* we try to get enough quota to write here, and let ldiskfs
+         * decide if it is out of quota or not b=14783 */
+        lquota_chkquota(mds_quota_interface_ref, obd,
+                        current->fsuid, gid, 1, &rec_pending);
+
         switch (type) {
         case S_IFREG:{
                 handle = fsfilt_start(obd, dir, FSFILT_OP_CREATE, NULL);
@@ -898,10 +914,7 @@ static int mds_reint_create(struct mds_update_record *rec, int offset,
                 LTIME_S(iattr.ia_ctime) = rec->ur_time;
                 LTIME_S(iattr.ia_mtime) = rec->ur_time;
                 iattr.ia_uid = current->fsuid;  /* set by push_ctxt already */
-                if (dir->i_mode & S_ISGID)
-                        iattr.ia_gid = dir->i_gid;
-                else
-                        iattr.ia_gid = current->fsgid;
+                iattr.ia_gid = gid;
                 iattr.ia_valid = ATTR_UID | ATTR_GID | ATTR_ATIME |
                         ATTR_MTIME | ATTR_CTIME;
 
@@ -928,7 +941,7 @@ static int mds_reint_create(struct mds_update_record *rec, int offset,
                 if (S_ISDIR(inode->i_mode)) {
                         struct lov_mds_md lmm;
                         int lmm_size = sizeof(lmm);
-                        rc = mds_get_md(obd, dir, &lmm, &lmm_size, 1);
+                        rc = mds_get_md(obd, dir, &lmm, &lmm_size, 1, 0);
                         if (rc > 0) {
                                 LOCK_INODE_MUTEX(inode);
                                 rc = fsfilt_set_md(obd, inode, handle,
@@ -948,6 +961,9 @@ static int mds_reint_create(struct mds_update_record *rec, int offset,
 
 cleanup:
         err = mds_finish_transno(mds, dir, handle, req, rc, 0, 0);
+        if (rec_pending)
+                lquota_pending_commit(mds_quota_interface_ref, obd,
+                                      current->fsuid, gid, 1);
 
         if (rc && created) {
                 /* Destroy the file we just created.  This should not need
@@ -1071,7 +1087,7 @@ int enqueue_ordered_locks(struct obd_device *obd, struct ldlm_res_id *p1_res_id,
                res_id[0]->name[0], res_id[1]->name[0]);
 
         flags = LDLM_FL_LOCAL_ONLY | LDLM_FL_ATOMIC_CB;
-        rc = ldlm_cli_enqueue_local(obd->obd_namespace, *res_id[0],
+        rc = ldlm_cli_enqueue_local(obd->obd_namespace, res_id[0],
                                     LDLM_IBITS, policies[0], lock_modes[0],
                                     &flags, ldlm_blocking_ast, 
                                     ldlm_completion_ast, NULL, NULL, 0, 
@@ -1086,7 +1102,7 @@ int enqueue_ordered_locks(struct obd_device *obd, struct ldlm_res_id *p1_res_id,
                 ldlm_lock_addref(handles[1], lock_modes[1]);
         } else if (res_id[1]->name[0] != 0) {
                 flags = LDLM_FL_LOCAL_ONLY | LDLM_FL_ATOMIC_CB;
-                rc = ldlm_cli_enqueue_local(obd->obd_namespace, *res_id[1],
+                rc = ldlm_cli_enqueue_local(obd->obd_namespace, res_id[1],
                                             LDLM_IBITS, policies[1],
                                             lock_modes[1], &flags, 
                                             ldlm_blocking_ast,
@@ -1197,7 +1213,7 @@ int enqueue_4ordered_locks(struct obd_device *obd,struct ldlm_res_id *p1_res_id,
                                 try_to_aggregate_locks(res_id[i], policies[i],
                                                        res_id[i+1], policies[i+1]);
                         rc = ldlm_cli_enqueue_local(obd->obd_namespace,
-                                                    *res_id[i], LDLM_IBITS,
+                                                    res_id[i], LDLM_IBITS,
                                                     policies[i], lock_modes[i],
                                                     &flags, ldlm_blocking_ast,
                                                     ldlm_completion_ast, NULL, 
@@ -1287,7 +1303,7 @@ static int mds_verify_child(struct obd_device *obd,
                         GOTO(cleanup, rc = 1);
                 }
 
-                rc = ldlm_cli_enqueue_local(obd->obd_namespace, *child_res_id, 
+                rc = ldlm_cli_enqueue_local(obd->obd_namespace, child_res_id, 
                                             LDLM_IBITS, child_policy, 
                                             child_mode, &flags, 
                                             ldlm_blocking_ast, 
@@ -1653,7 +1669,7 @@ static int mds_reint_unlink(struct mds_update_record *rec, int offset,
                         mds_pack_inode2fid(&body->fid1, child_inode);
                         mds_pack_inode2body(body, child_inode);
                         mds_pack_md(obd, req->rq_repmsg, offset + 1, body,
-                                    child_inode, MDS_PACK_MD_LOCK);
+                                    child_inode, MDS_PACK_MD_LOCK, 0);
                 }
         }
 
@@ -2229,7 +2245,7 @@ static int mds_reint_rename(struct mds_update_record *rec, int offset,
                         mds_pack_inode2fid(&body->fid1, new_inode);
                         mds_pack_inode2body(body, new_inode);
                         mds_pack_md(obd, req->rq_repmsg, offset + 1, body,
-                                    new_inode, MDS_PACK_MD_LOCK);
+                                    new_inode, MDS_PACK_MD_LOCK, 0);
                 }
         }
 
@@ -2348,6 +2364,9 @@ int mds_reint_rec(struct mds_update_record *rec, int offset,
         struct mds_obd *mds = &obd->u.mds;
         struct lvfs_run_ctxt saved;
         int rc;
+#ifdef CRAY_XT3
+        gid_t fsgid = rec->ur_uc.luc_fsgid;
+#endif
         ENTRY;
 
 #ifdef CRAY_XT3
@@ -2381,6 +2400,15 @@ int mds_reint_rec(struct mds_update_record *rec, int offset,
 #endif
 
         push_ctxt(&saved, &obd->obd_lvfs_ctxt, &rec->ur_uc);
+
+#ifdef CRAY_XT3
+        if (rec->ur_uc.luc_uce && fsgid != rec->ur_uc.luc_fsgid &&
+            in_group_p(fsgid)) {
+                rec->ur_uc.luc_fsgid = fsgid;
+                current->fsgid = saved.luc.luc_fsgid = fsgid;
+        }
+#endif
+
         rc = reinters[rec->ur_opcode] (rec, offset, req, lockh);
         pop_ctxt(&saved, &obd->obd_lvfs_ctxt, &rec->ur_uc);
 
