@@ -23,7 +23,7 @@
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
- 
+
 #ifndef EXPORT_SYMTAB
 # define EXPORT_SYMTAB
 #endif
@@ -41,42 +41,56 @@
 
 #include <obd_class.h>
 #include <lustre_dlm.h>
+#include <lprocfs_status.h>
 #include <lustre_log.h>
 #include <lustre_fsfilt.h>
 #include <lustre_disk.h>
+#include "mgc_internal.h"
 
+static int mgc_name2resid(char *name, int len, struct ldlm_res_id *res_id)
+{
+        __u64 resname = 0;
+
+        if (len > 8) {
+                CERROR("name too long: %s\n", name);
+                return -EINVAL;
+        }
+        if (len <= 0) {
+                CERROR("missing name: %s\n", name);
+                return -EINVAL;
+        }
+        memcpy(&resname, name, len);
+
+        memset(res_id, 0, sizeof(*res_id));
+
+        /* Always use the same endianness for the resid */
+        res_id->name[0] = cpu_to_le64(resname);
+        CDEBUG(D_MGC, "log %s to resid "LPX64"/"LPX64" (%.8s)\n", name,
+               res_id->name[0], res_id->name[1], (char *)&res_id->name[0]);
+        return 0;
+}
+
+int mgc_fsname2resid(char *fsname, struct ldlm_res_id *res_id)
+{
+        /* fsname is at most 8 chars long, maybe contain "-".
+         * e.g. "lustre", "CFS-000" */
+        return mgc_name2resid(fsname, strlen(fsname), res_id);
+}
+EXPORT_SYMBOL(mgc_fsname2resid);
 
 int mgc_logname2resid(char *logname, struct ldlm_res_id *res_id)
 {
         char *name_end;
         int len;
-        __u64 resname = 0;
-        
-        /* fsname is at most 8 chars long at the beginning of the logname
-           e.g. "lustre-MDT0001" or "lustre" */
-        name_end = strrchr(logname, '-');
-        if (name_end)
-                len = name_end - logname;
-        else
-                len = strlen(logname);
-        if (len > 8) {
-                CERROR("fsname too long: %s\n", logname);
-                return -EINVAL;
-        }
-        if (len <= 0) {
-                CERROR("missing fsname: %s\n", logname);
-                return -EINVAL;
-        }
-        memcpy(&resname, logname, len);
 
-        memset(res_id, 0, sizeof(*res_id));
-        /* Always use the same endianness for the resid */
-        res_id->name[0] = cpu_to_le64(resname);
-        CDEBUG(D_MGC, "log %s to resid "LPX64"/"LPX64" (%.8s)\n", logname,
-               res_id->name[0], res_id->name[1], (char *)&res_id->name[0]);
-        return 0;
+        /* logname consists of "fsname-nodetype".
+         * e.g. "lustre-MDT0001", "CFS-000-client" */
+        name_end = strrchr(logname, '-');
+        LASSERT(name_end);
+        len = name_end - logname;
+        return mgc_name2resid(logname, len, res_id);
 }
-EXPORT_SYMBOL(mgc_logname2resid);
+
 
 /********************** config llog list **********************/
 static struct list_head config_llog_list = LIST_HEAD_INIT(config_llog_list);
@@ -481,6 +495,7 @@ static int mgc_cleanup(struct obd_device *obd)
                 /* Only for the last mgc */
                 class_del_profiles();
 
+        lprocfs_obd_cleanup(obd);
         ptlrpcd_decref();
 
         rc = client_obd_cleanup(obd);
@@ -489,6 +504,7 @@ static int mgc_cleanup(struct obd_device *obd)
 
 static int mgc_setup(struct obd_device *obd, obd_count len, void *buf)
 {
+        struct lprocfs_static_vars lvars;
         int rc;
         ENTRY;
 
@@ -504,6 +520,9 @@ static int mgc_setup(struct obd_device *obd, obd_count len, void *buf)
                 GOTO(err_cleanup, rc);
         }
 
+        lprocfs_mgc_init_vars(&lvars);
+        lprocfs_obd_setup(obd, lvars.obd_vars);
+
         spin_lock(&config_list_lock);
         atomic_inc(&mgc_count);
         if (atomic_read(&mgc_count) == 1) {
@@ -767,33 +786,6 @@ static int mgc_set_mgs_param(struct obd_export *exp,
         RETURN(rc);
 }
 
-int mgc_reconnect_import(struct obd_import *imp)
-{
-        /* Force a new connect attempt */
-        ptlrpc_invalidate_import(imp);
-        /* Do a fresh connect next time by zeroing the handle */
-        ptlrpc_disconnect_import(imp, 1);
-        /* Wait for all invalidate calls to finish */
-        if (atomic_read(&imp->imp_inval_count) > 0) {
-                int rc;
-                struct l_wait_info lwi = LWI_INTR(LWI_ON_SIGNAL_NOOP, NULL);
-                rc = l_wait_event(imp->imp_recovery_waitq,
-                                  (atomic_read(&imp->imp_inval_count) == 0),
-                                  &lwi);
-                if (rc)
-                        CERROR("Interrupted, inval=%d\n", 
-                               atomic_read(&imp->imp_inval_count));
-        }
-
-        /* Allow reconnect attempts */
-        imp->imp_obd->obd_no_recov = 0;
-        /* Remove 'invalid' flag */
-        ptlrpc_activate_import(imp);
-        /* Attempt a new connect */
-        ptlrpc_recover_import(imp, NULL);
-        return 0;
-}
-
 int mgc_set_info_async(struct obd_export *exp, obd_count keylen,
                        void *key, obd_count vallen, void *val, 
                        struct ptlrpc_request_set *set)
@@ -832,7 +824,7 @@ int mgc_set_info_async(struct obd_export *exp, obd_count keylen,
                        ptlrpc_import_state_name(imp->imp_state));
                 /* Resurrect if we previously died */
                 if (imp->imp_invalid || value > 1) 
-                        mgc_reconnect_import(imp);
+                        ptlrpc_reconnect_import(imp);
                 RETURN(0);
         }
         /* FIXME move this to mgc_process_config */
@@ -876,7 +868,7 @@ int mgc_set_info_async(struct obd_export *exp, obd_count keylen,
         }
 
         RETURN(rc);
-}               
+}
 
 static int mgc_import_event(struct obd_device *obd,
                             struct obd_import *imp,
@@ -890,7 +882,6 @@ static int mgc_import_event(struct obd_device *obd,
         switch (event) {
         case IMP_EVENT_DISCON: 
                 /* MGC imports should not wait for recovery */
-                ptlrpc_invalidate_import(imp);
                 break;
         case IMP_EVENT_INACTIVE: 
                 break;
@@ -901,6 +892,8 @@ static int mgc_import_event(struct obd_device *obd,
         }
         case IMP_EVENT_ACTIVE: 
                 LCONSOLE_WARN("%s: Reactivating import\n", obd->obd_name);
+                /* Clearing obd_no_recov allows us to continue pinging */
+                obd->obd_no_recov = 0;
                 break;
         case IMP_EVENT_OCD:
                 break;
@@ -1136,8 +1129,9 @@ static int mgc_process_log(struct obd_device *mgc,
                                                    "\n", cld->cld_logname);
                                 GOTO(out_pop, rc = -ENOTCONN);
                         }
-                        LCONSOLE_WARN("Failed to get MGS log %s, using "
-                                      "local copy.\n", cld->cld_logname);
+                        CDEBUG(D_MGC, "Failed to get MGS log %s, using local "
+                                      "copy for now, will try to update later.\n",
+                               cld->cld_logname);
                 }
                 /* Now, whether we copied or not, start using the local llog.
                    If we failed to copy, we'll start using whatever the old 
