@@ -521,7 +521,7 @@ DECLARE_MUTEX(mgc_start_lock);
 static int lustre_start_mgc(struct super_block *sb)
 {
         struct lustre_handle mgc_conn = {0, };
-        struct obd_connect_data ocd = { 0 };
+        struct obd_connect_data *data = NULL;
         struct lustre_sb_info *lsi = s2lsi(sb);
         struct obd_device *obd;
         struct obd_export *exp;
@@ -699,10 +699,16 @@ static int lustre_start_mgc(struct super_block *sb)
                                 sizeof(recov_bk), &recov_bk, NULL);
         if (rc)
                 /* nonfatal */
-                CERROR("can't set %s %d\n", KEY_INIT_RECOV_BACKUP, rc);
+                CWARN("can't set %s %d\n", KEY_INIT_RECOV_BACKUP, rc);
 
+        OBD_ALLOC_PTR(data);
+        if (data == NULL)
+                GOTO(out, rc = -ENOMEM);
+        data->ocd_connect_flags = OBD_CONNECT_VERSION | OBD_CONNECT_AT;
+        data->ocd_version = LUSTRE_VERSION_CODE;
         /* We connect to the MGS at setup, and don't disconnect until cleanup */
-        rc = obd_connect(&mgc_conn, obd, &(obd->obd_uuid), &ocd);
+        rc = obd_connect(&mgc_conn, obd, &(obd->obd_uuid), data, NULL);
+        OBD_FREE_PTR(data);
         if (rc) {
                 CERROR("connect failed %d\n", rc);
                 GOTO(out, rc);
@@ -749,13 +755,17 @@ static int lustre_stop_mgc(struct super_block *sb)
                 GOTO(out, rc = -EBUSY);
         }
 
-        /* MGC should disconnect nicely so MGS won't print eviction messages */
-        obd->obd_force = (lsi->lsi_flags & LSI_UMOUNT_FORCE) != 0;
-        /* The MGC has no recoverable data in any case. */
+        /* The MGC has no recoverable data in any case. 
+         * force shotdown set in umount_begin */
         obd->obd_no_recov = 1;
 
-        if (obd->u.cli.cl_mgc_mgsexp)
-                obd_disconnect(obd->u.cli.cl_mgc_mgsexp);
+        if (obd->u.cli.cl_mgc_mgsexp) {
+                /* An error is not fatal, if we are unable to send the
+                   disconnect mgs ping evictor cleans up the export */
+                rc = obd_disconnect(obd->u.cli.cl_mgc_mgsexp);
+                if (rc)
+                        CDEBUG(D_MOUNT, "disconnect failed %d\n", rc);
+        }
 
         /* Save the obdname for cleaning the nid uuids, which are
            obdname_XX */
@@ -950,10 +960,8 @@ int server_register_target(struct super_block *sb)
         rc = obd_set_info_async(mgc->u.cli.cl_mgc_mgsexp,
                                 strlen("register_target"), "register_target",
                                 sizeof(*mti), mti, NULL);
-        if (rc) {
-                CERROR("registration with the MGS failed (%d)\n", rc);
+        if (rc)
                 GOTO(out, rc);
-        }
 
         /* Always update our flags */
         ldd->ldd_flags = mti->mti_flags & ~LDD_F_REWRITE_LDD;
@@ -1065,6 +1073,9 @@ static int server_start_targets(struct super_block *sb, struct vfsmount *mnt)
                                    lsi->lsi_ldd->ldd_svname);
                 GOTO(out_mgc, rc);
         }
+        /* non-fatal error of registeration with MGS */
+        if (rc)
+                CDEBUG(D_MOUNT, "Cannot register with MGS: %d\n", rc);
 
         /* Let the target look up the mount using the target's name
            (we can't pass the sb or mnt through class_process_config.) */
@@ -1203,6 +1214,7 @@ static struct vfsmount *server_kernel_mount(struct super_block *sb)
         struct vfsmount *mnt;
         char *options = NULL;
         unsigned long page, s_flags;
+        struct page *__page;
         int rc;
         ENTRY;
 
@@ -1256,9 +1268,10 @@ static struct vfsmount *server_kernel_mount(struct super_block *sb)
         /* Done with our pre-mount, now do the real mount. */
 
         /* Glom up mount options */
-        page = __get_free_page(GFP_KERNEL);
-        if (!page)
+        OBD_PAGE_ALLOC(__page, CFS_ALLOC_STD);
+        if (!__page)
                 GOTO(out_free, rc = -ENOMEM);
+        page = (unsigned long)cfs_page_address(__page);
 
         options = (char *)page;
         memset(options, 0, CFS_PAGE_SIZE);
@@ -1280,7 +1293,7 @@ static struct vfsmount *server_kernel_mount(struct super_block *sb)
                MT_STR(ldd), lmd->lmd_dev, options);
         mnt = ll_kern_mount(MT_STR(ldd), s_flags, lmd->lmd_dev,
                             (void *)options);
-        free_page(page);
+        OBD_PAGE_FREE(__page);
         if (IS_ERR(mnt)) {
                 rc = PTR_ERR(mnt);
                 CERROR("ll_kern_mount failed: rc = %d\n", rc);
@@ -1312,12 +1325,13 @@ static void server_wait_finished(struct vfsmount *mnt)
 
                 /* Wait for a bit */
                 retries -= 5;
-                lwi = LWI_TIMEOUT(cfs_time_seconds(5), NULL, NULL);
+                lwi = LWI_TIMEOUT(5 * HZ, NULL, NULL);
                 l_wait_event(waitq, 0, &lwi);
         }
-        if (atomic_read(&mnt->mnt_count) > 1)
+        if (atomic_read(&mnt->mnt_count) > 1) {
                 CERROR("Mount %p is still busy (%d refs), giving up.\n",
                        mnt, atomic_read(&mnt->mnt_count));
+        }
 }
 
 static void server_put_super(struct super_block *sb)
@@ -1361,7 +1375,7 @@ static void server_put_super(struct super_block *sb)
                         if (lsi->lsi_flags & LSI_UMOUNT_FAILOVER)
                                 obd->obd_fail = 1;
                         /* We can't seem to give an error return code
-                           to .put_super, so we better make sure we clean up! */
+                         * to .put_super, so we better make sure we clean up! */
                         obd->obd_force = 1;
                         class_manual_cleanup(obd);
                 } else {
@@ -1473,11 +1487,24 @@ static int server_statfs (struct dentry *dentry, struct kstatfs *buf)
         RETURN(0);
 }
 
+static int server_show_options(struct seq_file *seq, struct vfsmount *vfsmnt)
+{
+        struct vfsmount *mnt = s2lsi(vfsmnt->mnt_sb)->lsi_srv_mnt;
+        ENTRY;
+
+        if (mnt && mnt->mnt_sb && mnt->mnt_sb->s_op->show_options) {
+                int rc = mnt->mnt_sb->s_op->show_options(seq, mnt);
+                RETURN(rc);
+        }
+        RETURN(0);
+}
+
 static struct super_operations server_ops =
 {
         .put_super      = server_put_super,
         .umount_begin   = server_umount_begin, /* umount -f */
         .statfs         = server_statfs,
+        .show_options   = server_show_options,
 };
 
 #define log2(n) ffz(~(n))
