@@ -5,6 +5,8 @@
 #ifndef OST_INTERNAL_H
 #define OST_INTERNAL_H
 
+#define OSS_SERVICE_WATCHDOG_FACTOR 2000
+
 /*
  * tunables for per-thread page pool (bug 5137)
  */
@@ -31,10 +33,19 @@ struct ost_thread_local_cache {
 
 struct ost_thread_local_cache *ost_tls(struct ptlrpc_request *r);
 
-#define OSS_DEF_CREATE_THREADS  1UL
+#define OSS_DEF_CREATE_THREADS  2UL
 #define OSS_MAX_CREATE_THREADS 16UL
 
 /* Quota stuff */
 extern quota_interface_t *quota_interface;
 
+#ifdef LPROCFS
+void lprocfs_ost_init_vars(struct lprocfs_static_vars *lvars);
+#else
+static void lprocfs_ost_init_vars(struct lprocfs_static_vars *lvars)
+{
+        memset(lvars, 0, sizeof(*lvars));
+}
+#endif
+
 #endif /* OST_INTERNAL_H */
