@@ -25,8 +25,12 @@ fi
 [ "$DEBUG_OFF" ] || DEBUG_OFF="eval sysctl -w lnet.debug=\"$DEBUG_LVL\""
 [ "$DEBUG_ON" ] || DEBUG_ON="eval sysctl -w lnet.debug=0x33f0484"
 
+export TESTSUITE_LIST="RUNTESTS SANITY DBENCH BONNIE IOZONE FSX SANITYN LFSCK LIBLUSTRE REPLAY_SINGLE CONF_SANITY RECOVERY_SMALL REPLAY_OST_SINGLE REPLAY_DUAL INSANITY SANITY_QUOTA"
+
 if [ "$ACC_SM_ONLY" ]; then
-    export RUNTESTS="no" SANITY="no" DBENCH="no" BONNIE="no" IOZONE="no" FSX="no" SANITYN="no" LFSCK="no" LIBLUSTRE="no" REPLAY_SINGLE="no" CONF_SANITY="no" RECOVERY_SMALL="no" REPLAY_OST_SINGLE="no" REPLAY_DUAL="no" INSANITY="no" SANITY_QUOTA="no"
+    for O in $TESTSUITE_LIST; do
+	export ${O}="no"
+    done
     for O in $ACC_SM_ONLY; do
 	O=`echo $O | tr "[:lower:]" "[:upper:]"`
 	export ${O}="yes"
@@ -59,6 +63,11 @@ title() {
 for NAME in $CONFIGS; do
 	export NAME MOUNT START CLEAN
 	. $LUSTRE/tests/cfg/$NAME.sh
+
+	if [ ! -f /lib/modules/$(uname -r)/kernel/fs/lustre/mds.ko -a \
+	    ! -f `dirname $0`/../mds/mds.ko ]; then
+	    export CLIENTMODSONLY=true
+	fi
 	
 	assert_env mds_HOST MDS_MKFS_OPTS MDSDEV
 	assert_env ost_HOST OST_MKFS_OPTS OSTCOUNT
@@ -71,72 +80,96 @@ for NAME in $CONFIGS; do
 		bash runtests
 		$CLEANUP
 		$SETUP
+		RUNTESTS="done"
 	fi
 
 	if [ "$SANITY" != "no" ]; then
 	        title sanity
-		bash sanity.sh
+		MOUNT2="" bash sanity.sh
 		$CLEANUP
 		$SETUP
+		SANITY="done"
 	fi
 
 	which dbench > /dev/null 2>&1 || DBENCH=no
 	if [ "$DBENCH" != "no" ]; then
 	        title dbench
+		DBENCHDIR=$MOUNT/$HOSTNAME
+		mkdir -p $DBENCHDIR
 		SPACE=`df -P $MOUNT | tail -n 1 | awk '{ print $4 }'`
 		DB_THREADS=$((SPACE / 50000))
 		[ $THREADS -lt $DB_THREADS ] && DB_THREADS=$THREADS
 
 		$DEBUG_OFF
-		bash rundbench 1
-		$DEBUG_ON
-		$CLEANUP
-		$SETUP
-		if [ $DB_THREADS -gt 1 ]; then
-			$DEBUG_OFF
-			bash rundbench $DB_THREADS
+		myUID=$RUNAS_ID
+		myRUNAS=$RUNAS
+		FAIL_ON_ERROR=false check_runas_id_ret $myUID $myRUNAS || { myRUNAS="" && myUID=$UID; }
+		chown $myUID:$myUID $DBENCHDIR
+		duration=""
+		[ "$SLOW" = "no" ] && duration=" -t 120"
+		if [ "$SLOW" != "no" -o $DB_THREADS -eq 1 ]; then
+			DIR=$DBENCHDIR $myRUNAS bash rundbench 1 $duration
 			$DEBUG_ON
 			$CLEANUP
 			$SETUP
 		fi
-		rm -f /mnt/lustre/`hostname`/client.txt
+		if [ $DB_THREADS -gt 1 ]; then
+			$DEBUG_OFF
+			DIR=$DBENCHDIR $myRUNAS bash rundbench $DB_THREADS $duration
+			$DEBUG_ON
+			$CLEANUP
+			$SETUP
+		fi
+		rm -rf $DBENCHDIR
+		DBENCH="done"
 	fi
 
-	chown $UID $MOUNT
 	which bonnie++ > /dev/null 2>&1 || BONNIE=no
 	if [ "$BONNIE" != "no" ]; then
 	        title bonnie
-		mkdir -p $MOUNT/d0.bonnie
-		$LFS setstripe -c -1 $MOUNT/d0.bonnie
+		BONDIR=$MOUNT/d0.bonnie
+		mkdir -p $BONDIR
+		$LFS setstripe -c -1 $BONDIR
 		sync
 		MIN=`cat /proc/fs/lustre/osc/*/kbytesavail | sort -n | head -n1`
 		SPACE=$(( OSTCOUNT * MIN ))
 		[ $SPACE -lt $SIZE ] && SIZE=$((SPACE * 3 / 4))
 		log "min OST has ${MIN}kB available, using ${SIZE}kB file size"
 		$DEBUG_OFF
-		BONFILE=$MOUNT/d0.bonnie
-		bonnie++ -f -r 0 -s$((SIZE / 1024)) -n 10 -u$UID -d$BONFILE
+		myUID=$RUNAS_ID
+		myRUNAS=$RUNAS
+		FAIL_ON_ERROR=false check_runas_id_ret $myUID $myRUNAS || { myRUNAS="" && myUID=$UID; }
+		chown $myUID:$myUID $BONDIR		
+		$myRUNAS bonnie++ -f -r 0 -s$((SIZE / 1024)) -n 10 -u$myUID:$myUID -d$BONDIR
 		$DEBUG_ON
 		$CLEANUP
 		$SETUP
+		BONNIE="done"
 	fi
 
 	export O_DIRECT
+	[ "$SLOW" = "no" ] && export IOZONE=no # 5 minutes
+
 	which iozone > /dev/null 2>&1 || IOZONE=no
 	if [ "$IOZONE" != "no" ]; then
 	        title iozone
-		mkdir -p $MOUNT/d0.iozone
-		$LFS setstripe -c -1 $MOUNT/d0.iozone
+		IOZDIR=$MOUNT/d0.iozone
+		mkdir -p $IOZDIR
+		$LFS setstripe -c -1 $IOZDIR
 		sync
 		MIN=`cat /proc/fs/lustre/osc/*/kbytesavail | sort -n | head -n1`
 		SPACE=$(( OSTCOUNT * MIN ))
 		[ $SPACE -lt $SIZE ] && SIZE=$((SPACE * 3 / 4))
 		log "min OST has ${MIN}kB available, using ${SIZE}kB file size"
 		IOZONE_OPTS="-i 0 -i 1 -i 2 -e -+d -r $RSIZE -s $SIZE"
-		IOZFILE="$MOUNT/d0.iozone/iozone"
+		IOZFILE="$IOZDIR/iozone"
 		# $SPACE was calculated with all OSTs
 		$DEBUG_OFF
-		iozone $IOZONE_OPTS -f $IOZFILE
+		myUID=$RUNAS_ID
+		myRUNAS=$RUNAS
+		FAIL_ON_ERROR=false check_runas_id_ret $myUID $myRUNAS || { myRUNAS="" && myUID=$UID; }
+		chown $myUID:$myUID $IOZDIR
+		$myRUNAS iozone $IOZONE_OPTS -f $IOZFILE
 		$DEBUG_ON
 		$CLEANUP
 		$SETUP
@@ -151,7 +184,8 @@ for NAME in $CONFIGS; do
 		fi
 		if [ "$O_DIRECT" != "no" -a "$IOZONE_DIR" != "no" ]; then
 			$DEBUG_OFF
-			iozone -I $IOZONE_OPTS $IOZFILE.odir
+			# cd TMP to have write permission for tmp file iozone writes
+			( cd $TMP && $myRUNAS iozone -I $IOZONE_OPTS $IOZFILE.odir )
 			$DEBUG_ON
 			$CLEANUP
 			$SETUP
@@ -162,15 +196,15 @@ for NAME in $CONFIGS; do
 		[ $THREADS -lt $IOZ_THREADS ] && IOZ_THREADS=$THREADS
 		IOZVER=`iozone -v | awk '/Revision:/ {print $3}' | tr -d .`
 		if [ "$IOZ_THREADS" -gt 1 -a "$IOZVER" -ge 3145 ]; then
-			$LFS setstripe -c 1 $MOUNT/d0.iozone
+			$LFS setstripe -c 1 $IOZDIR
 			$DEBUG_OFF
 			THREAD=1
 			IOZFILE="-F "
 			while [ $THREAD -le $IOZ_THREADS ]; do
-				IOZFILE="$IOZFILE $MOUNT/iozone.$THREAD"
+				IOZFILE="$IOZFILE $IOZDIR/iozone.$THREAD"
 				THREAD=$((THREAD + 1))
 			done
-			iozone $IOZONE_OPTS -t $IOZ_THREADS $IOZFILE
+			$myRUNAS iozone $IOZONE_OPTS -t $IOZ_THREADS $IOZFILE
 			$DEBUG_ON
 			$CLEANUP
 			$SETUP
@@ -178,6 +212,7 @@ for NAME in $CONFIGS; do
 			VER=`iozone -v | awk '/Revision:/ { print $3 }'`
 			echo "iozone $VER too old for multi-thread test"
 		fi
+		IOZONE="done"
 	fi
 
 	if [ "$FSX" != "no" ]; then
@@ -190,6 +225,7 @@ for NAME in $CONFIGS; do
 		$DEBUG_ON
 		$CLEANUP
 		$SETUP
+		FSX="done"
 	fi	
 
 	if [ "$SANITYN" != "no" ]; then
@@ -205,6 +241,7 @@ for NAME in $CONFIGS; do
 		$DEBUG_ON
 		$CLEANUP
 		$SETUP
+		SANITYN="done"
 	fi
 
 	if [ "$LFSCK" != "no" -a -x /usr/sbin/lfsck ]; then
@@ -212,8 +249,9 @@ for NAME in $CONFIGS; do
 		E2VER=`e2fsck -V 2>&1 | head -n 1 | cut -d' ' -f 2`
 		if grep -q obdfilter /proc/fs/lustre/devices; then
 			if [ `echo $E2VER | cut -d. -f2` -ge 39 ] && \
-			   [ "`echo $E2VER | grep cfs`" ]; then
-			   	bash lfscktest.sh
+			   [ "`echo $E2VER | grep cfs`" -o \
+				"`echo $E2VER | grep sun`" ]; then
+			   		bash lfscktest.sh
 			else
 				e2fsck -V
 				echo "e2fsck does not support lfsck, skipping"
@@ -221,8 +259,10 @@ for NAME in $CONFIGS; do
 		else
 			echo "remote OST, skipping test"
 		fi
+		LFSCK="done"
 	fi
 
+	[ "$NETTYPE" = "tcp" -o "$NETTYPE" = "ptl" ] || LIBLUSTRE=no # bug 15660
 	if [ "$LIBLUSTRE" != "no" ]; then
 	        title liblustre
 		assert_env MGSNID MOUNT2
@@ -232,7 +272,12 @@ for NAME in $CONFIGS; do
 		[ -f /etc/modprobe.conf ] && MODPROBECONF=/etc/modprobe.conf
 		[ -f /etc/modprobe.d/Lustre ] && MODPROBECONF=/etc/modprobe.d/Lustre
 
-		LNETOPTS="$(awk '/^options lnet/ { print $0}' $MODPROBECONF | sed 's/^options lnet //g') accept=all" MDS_MOUNT_OPTS="${MDS_MOUNT_OPTS},noacl" $SETUP
+		LNETOPTS="$(awk '/^options lnet/ { print $0}' $MODPROBECONF | \
+			sed 's/^options lnet //g') accept=all" \
+			MDS_MOUNT_OPTS=$(echo $MDS_MOUNT_OPTS | sed 's/^[ \t]*//;s/[ \t]*$//') \
+			MDS_MOUNT_OPTS="${MDS_MOUNT_OPTS},noacl" \
+			MDS_MOUNT_OPTS=${MDS_MOUNT_OPTS/#,/-o } \
+			$SETUP
 		export LIBLUSTRE_MOUNT_POINT=$MOUNT2
 		export LIBLUSTRE_MOUNT_TARGET=$MGSNID:/$FSNAME
 		export LIBLUSTRE_TIMEOUT=`cat /proc/sys/lustre/timeout`
@@ -244,6 +289,7 @@ for NAME in $CONFIGS; do
 		fi
 		$CLEANUP
 		#$SETUP
+		LIBLUSTRE="done"
 	fi
 
 	$CLEANUP
@@ -252,36 +298,43 @@ done
 if [ "$REPLAY_SINGLE" != "no" ]; then
         title replay-single
 	bash replay-single.sh
+	REPLAY_SINGLE="done"
 fi
 
 if [ "$CONF_SANITY" != "no" ]; then
         title conf-sanity
         bash conf-sanity.sh
+        CONF_SANITY="done"
 fi
 
 if [ "$RECOVERY_SMALL" != "no" ]; then
         title recovery-small
         bash recovery-small.sh
+        RECOVERY_SMALL="done"
 fi
 
 if [ "$REPLAY_OST_SINGLE" != "no" ]; then
         title replay-ost-single
         bash replay-ost-single.sh
+        REPLAY_OST_SINGLE="done"
 fi
 
 if [ "$REPLAY_DUAL" != "no" ]; then
         title replay-dual
         bash replay-dual.sh
+        REPLAY_DUAL="done"
 fi
 
 if [ "$INSANITY" != "no" ]; then
         title insanity
         bash insanity.sh -r
+        INSANITY="done"
 fi
 
 if [ "$SANITY_QUOTA" != "no" ]; then
         title sanity-quota
         bash sanity-quota.sh
+        SANITY_QUOTA="done"
 fi
 
 
@@ -289,5 +342,5 @@ RC=$?
 title FINISHED
 echo "Finished at `date` in $((`date +%s` - $STARTTIME))s"
 echo "Tests ran: $RANTEST"
-
+print_summary
 echo "$0: completed with rc $RC" && exit $RC
