@@ -33,8 +33,8 @@ fi
 export LANG=C LC_LANG=C # for "No space left on device" message
 
 # make sure we stripe over all OSTs to avoid OOS on only a subset of OSTs
-$LFS setstripe $OOS 65536 -1 $STRIPECOUNT
-$LFS setstripe $OOS2 65536 -1 $STRIPECOUNT
+$LFS setstripe $OOS -c $STRIPECOUNT
+$LFS setstripe $OOS2 -c $STRIPECOUNT
 dd if=/dev/zero of=$OOS count=$((3 * $ORIGFREE / 4 + 100)) bs=1k 2>> $LOG &
 DDPID=$!
 if dd if=/dev/zero of=$OOS2 count=$((3*$ORIGFREE/4 + 100)) bs=1k 2>> $LOG2; then
