@@ -11,6 +11,10 @@ LUSTRE=${LUSTRE:-`dirname $0`/..}
 init_test_env $@
 . ${CONFIG:=$LUSTRE/tests/cfg/$NAME.sh}
 
+# also long tests: 19, 21a, 21e, 21f, 23, 27
+#                                   1  2.5  2.5    4    4          (min)"
+[ "$SLOW" = "no" ] && EXCEPT_SLOW="17  26a  26b    50   51     57"
+
 build_test_filter
 
 # Allow us to override the setup if we already have a mounted system by
@@ -19,6 +23,7 @@ SETUP=${SETUP:-""}
 CLEANUP=${CLEANUP:-""}
 
 cleanup_and_setup_lustre
+rm -rf $DIR/[df][0-9]*
 
 test_1() {
     drop_request "mcreate $MOUNT/1"  || return 1
@@ -123,10 +128,9 @@ test_12(){
     $LCTL mark multiop $MOUNT/$tfile OS_c 
     do_facet mds "sysctl -w lustre.fail_loc=0x115"
     clear_failloc mds $((TIMEOUT * 2)) &
-    multiop $MOUNT/$tfile OS_c  &
+    multiop_bg_pause $MOUNT/$tfile OS_c || return 1
     PID=$!
 #define OBD_FAIL_MDS_CLOSE_NET           0x115
-    sleep 2
     kill -USR1 $PID
     echo "waiting for multiop $PID"
     wait $PID || return 2
@@ -164,17 +168,13 @@ test_15() {
 }
 run_test 15 "failed open (-ENOMEM)"
 
-READ_AHEAD=`cat $LPROC/llite/*/max_read_ahead_mb | head -n 1`
+READ_AHEAD=`lctl get_param -n llite.*.max_read_ahead_mb | head -n 1`
 stop_read_ahead() {
-   for f in $LPROC/llite/*/max_read_ahead_mb; do 
-      echo 0 > $f
-   done
+   lctl set_param -n llite.*.max_read_ahead_mb 0
 }
 
 start_read_ahead() {
-   for f in $LPROC/llite/*/max_read_ahead_mb; do 
-      echo $READ_AHEAD > $f
-   done
+   lctl set_param -n llite.*.max_read_ahead_mb $READ_AHEAD
 }
 
 test_16() {
@@ -196,6 +196,14 @@ test_16() {
 run_test 16 "timeout bulk put, don't evict client (2732)"
 
 test_17() {
+    local at_max_saved=0
+
+    # With adaptive timeouts, bulk_get won't expire until adaptive_timeout_max
+    if at_is_valid && at_is_enabled; then
+        at_max_saved=$(at_max_get ost1)
+        at_max_set $TIMEOUT ost1
+    fi
+
     # OBD_FAIL_PTLRPC_BULK_GET_NET 0x0503 | OBD_FAIL_ONCE
     # OST bulk will time out here, client retries
     do_facet ost1 sysctl -w lustre.fail_loc=0x80000503
@@ -203,12 +211,16 @@ test_17() {
     do_facet client cp $SAMPLE_FILE $DIR/$tfile
     sync
 
-    sleep $TIMEOUT
+    # with AT, client will wait adaptive_max*factor+net_latency before
+    # expiring the req, hopefully timeout*2 is enough
+    sleep $(($TIMEOUT*2))
+
     do_facet ost1 sysctl -w lustre.fail_loc=0
     do_facet client "df $DIR"
     # expect cmp to succeed, client resent bulk
     do_facet client "cmp $SAMPLE_FILE $DIR/$tfile" || return 3
     do_facet client "rm $DIR/$tfile" || return 4
+    [ $at_max_saved -ne 0 ] && $(at_max_set $at_max_saved ost1)
     return 0
 }
 run_test 17 "timeout bulk get, don't evict client (2732)"
@@ -223,11 +235,11 @@ test_18a() {
     pgcache_empty || return 1
 
     # 1 stripe on ost2
-    lfs setstripe $f $((128 * 1024)) 1 1
+    lfs setstripe $f -s $((128 * 1024)) -i 1 -c 1
 
     do_facet client cp $SAMPLE_FILE $f
     sync
-    local osc2dev=`grep ${ost2_svc}-osc- $LPROC/devices | awk '{print $1}'`
+    local osc2dev=`lctl get_param -n devices | grep ${ost2_svc}-osc- | awk '{print $1}'`
     $LCTL --device $osc2dev deactivate || return 3
     # my understanding is that there should be nothing in the page
     # cache after the client reconnects?     
@@ -248,12 +260,13 @@ test_18b() {
     pgcache_empty || return 1
 
     # shouldn't have to set stripe size of count==1
-    lfs setstripe $f $((128 * 1024)) 0 1
-    lfs setstripe $f2 $((128 * 1024)) 0 1
+    lfs setstripe $f -s $((128 * 1024)) -i 0 -c 1
+    lfs setstripe $f2 -s $((128 * 1024)) -i 0 -c 1
 
     do_facet client cp $SAMPLE_FILE $f
     sync
     ost_evict_client
+
     # force reconnect
     df $MOUNT > /dev/null 2>&1
     sleep 2
@@ -266,6 +279,37 @@ test_18b() {
 }
 run_test 18b "eviction and reconnect clears page cache (2766)"
 
+test_18c() {
+    do_facet client mkdir -p $MOUNT/$tdir
+    f=$MOUNT/$tdir/$tfile
+    f2=$MOUNT/$tdir/${tfile}-2
+
+    cancel_lru_locks osc
+    pgcache_empty || return 1
+
+    # shouldn't have to set stripe size of count==1
+    lfs setstripe $f -s $((128 * 1024)) -i 0 -c 1
+    lfs setstripe $f2 -s $((128 * 1024)) -i 0 -c 1
+
+    do_facet client cp $SAMPLE_FILE $f
+    sync
+    ost_evict_client
+
+    # OBD_FAIL_OST_CONNECT_NET2
+    # lost reply to connect request
+    do_facet ost1 sysctl -w lustre.fail_loc=0x80000225
+    # force reconnect
+    df $MOUNT > /dev/null 2>&1
+    sleep 2
+    # my understanding is that there should be nothing in the page
+    # cache after the client reconnects?     
+    rc=0
+    pgcache_empty || rc=2
+    rm -f $f $f2
+    return $rc
+}
+run_test 18c "Dropped connect reply after eviction handing (14755)"
+
 test_19a() {
     f=$MOUNT/$tfile
     do_facet client mcreate $f        || return 1
@@ -294,9 +338,9 @@ run_test 19b "test expired_lock_main on ost (2867)"
 
 test_20a() {	# bug 2983 - ldlm_handle_enqueue cleanup
 	mkdir -p $DIR/$tdir
-	multiop $DIR/$tdir/${tfile} O_wc &
+	lfs setstripe $DIR/$tdir/${tfile} -i 0 -c 1
+	multiop_bg_pause $DIR/$tdir/${tfile} O_wc || return 1
 	MULTI_PID=$!
-	sleep 1
 	cancel_lru_locks osc
 #define OBD_FAIL_LDLM_ENQUEUE_EXTENT_ERR 0x308
 	do_facet ost1 sysctl -w lustre.fail_loc=0x80000308
@@ -309,7 +353,7 @@ run_test 20a "ldlm_handle_enqueue error (should return error)"
 
 test_20b() {	# bug 2986 - ldlm_handle_enqueue error during open
 	mkdir -p $DIR/$tdir
-	touch $DIR/$tdir/${tfile}
+	lfs setstripe $DIR/$tdir/${tfile} -i 0 -c 1
 	cancel_lru_locks osc
 #define OBD_FAIL_LDLM_ENQUEUE_EXTENT_ERR 0x308
 	do_facet ost1 sysctl -w lustre.fail_loc=0x80000308
@@ -321,7 +365,7 @@ run_test 20b "ldlm_handle_enqueue error (should return error)"
 test_21a() {
        mkdir -p $DIR/$tdir-1
        mkdir -p $DIR/$tdir-2
-       multiop $DIR/$tdir-1/f O_c &
+       multiop_bg_pause $DIR/$tdir-1/f O_c || return 1
        close_pid=$!
 
        do_facet mds "sysctl -w lustre.fail_loc=0x80000129"
@@ -347,7 +391,7 @@ run_test 21a "drop close request while close and open are both in flight"
 test_21b() {
        mkdir -p $DIR/$tdir-1
        mkdir -p $DIR/$tdir-2
-       multiop $DIR/$tdir-1/f O_c &
+       multiop_bg_pause $DIR/$tdir-1/f O_c || return 1
        close_pid=$!
 
        do_facet mds "sysctl -w lustre.fail_loc=0x80000107"
@@ -370,7 +414,7 @@ run_test 21b "drop open request while close and open are both in flight"
 test_21c() {
        mkdir -p $DIR/$tdir-1
        mkdir -p $DIR/$tdir-2
-       multiop $DIR/$tdir-1/f O_c &
+       multiop_bg_pause $DIR/$tdir-1/f O_c || return 1
        close_pid=$!
 
        do_facet mds "sysctl -w lustre.fail_loc=0x80000107"
@@ -396,7 +440,7 @@ run_test 21c "drop both request while close and open are both in flight"
 test_21d() {
        mkdir -p $DIR/$tdir-1
        mkdir -p $DIR/$tdir-2
-       multiop $DIR/$tdir-1/f O_c &
+       multiop_bg_pause $DIR/$tdir-1/f O_c || return 1
        pid=$!
 
        do_facet mds "sysctl -w lustre.fail_loc=0x80000129"
@@ -420,7 +464,7 @@ run_test 21d "drop close reply while close and open are both in flight"
 test_21e() {
        mkdir -p $DIR/$tdir-1
        mkdir -p $DIR/$tdir-2
-       multiop $DIR/$tdir-1/f O_c &
+       multiop_bg_pause $DIR/$tdir-1/f O_c || return 1
        pid=$!
 
        do_facet mds "sysctl -w lustre.fail_loc=0x80000119"
@@ -442,7 +486,7 @@ run_test 21e "drop open reply while close and open are both in flight"
 test_21f() {
        mkdir -p $DIR/$tdir-1
        mkdir -p $DIR/$tdir-2
-       multiop $DIR/$tdir-1/f O_c &
+       multiop_bg_pause $DIR/$tdir-1/f O_c || return 1
        pid=$!
 
        do_facet mds "sysctl -w lustre.fail_loc=0x80000119"
@@ -465,7 +509,7 @@ run_test 21f "drop both reply while close and open are both in flight"
 test_21g() {
        mkdir -p $DIR/$tdir-1
        mkdir -p $DIR/$tdir-2
-       multiop $DIR/$tdir-1/f O_c &
+       multiop_bg_pause $DIR/$tdir-1/f O_c || return 1
        pid=$!
 
        do_facet mds "sysctl -w lustre.fail_loc=0x80000119"
@@ -488,7 +532,7 @@ run_test 21g "drop open reply and close request while close and open are both in
 test_21h() {
        mkdir -p $DIR/$tdir-1
        mkdir -p $DIR/$tdir-2
-       multiop $DIR/$tdir-1/f O_c &
+       multiop_bg_pause $DIR/$tdir-1/f O_c || return 1
        pid=$!
 
        do_facet mds "sysctl -w lustre.fail_loc=0x80000107"
@@ -532,7 +576,7 @@ test_22() {
 run_test 22 "drop close request and do mknod"
 
 test_23() { #b=4561
-    multiop $DIR/$tfile O_c &
+    multiop_bg_pause $DIR/$tfile O_c || return 1
     pid=$!
     # give a chance for open
     sleep 5
@@ -549,11 +593,9 @@ run_test 23 "client hang when close a file after mds crash"
 test_24() {	# bug 2248 - eviction fails writeback but app doesn't see it
 	mkdir -p $DIR/$tdir
 	cancel_lru_locks osc
-	multiop $DIR/$tdir/$tfile Owy_wyc &
+	multiop_bg_pause $DIR/$tdir/$tfile Owy_wyc || return 1
 	MULTI_PID=$!
-	sleep 0.500s
 	ost_evict_client
-	sleep 0.500s
 	kill -USR1 $MULTI_PID
 	wait $MULTI_PID
 	rc=$?
@@ -563,12 +605,12 @@ test_24() {	# bug 2248 - eviction fails writeback but app doesn't see it
 }
 run_test 24 "fsync error (should return error)"
 
-test_26() {      # bug 5921 - evict dead exports by pinger
+test_26a() {      # was test_26 bug 5921 - evict dead exports by pinger
 # this test can only run from a client on a separate node.
-	remote_ost || skip "local OST" && return
-	remote_mds || skip "local MDS" && return
-	OST_FILE=$LPROC/obdfilter/${ost1_svc}/num_exports
-        OST_EXP="`do_facet ost1 cat $OST_FILE`"
+	remote_ost || { skip "local OST" && return 0; }
+	remote_mds || { skip "local MDS" && return 0; }
+	OST_FILE=obdfilter.${ost1_svc}.num_exports
+        OST_EXP="`do_facet ost1 lctl get_param -n $OST_FILE`"
 	OST_NEXP1=`echo $OST_EXP | cut -d' ' -f2`
 	echo starting with $OST_NEXP1 OST exports
 # OBD_FAIL_PTLRPC_DROP_RPC 0x505
@@ -578,32 +620,32 @@ test_26() {      # bug 5921 - evict dead exports by pinger
 	# might have to wait for the next ping.
 	echo Waiting for $(($TIMEOUT * 4)) secs
 	sleep $(($TIMEOUT * 4))
-        OST_EXP="`do_facet ost1 cat $OST_FILE`"
+        OST_EXP="`do_facet ost1 lctl get_param -n $OST_FILE`"
 	OST_NEXP2=`echo $OST_EXP | cut -d' ' -f2`
 	echo ending with $OST_NEXP2 OST exports
 	do_facet client sysctl -w lustre.fail_loc=0x0
         [ $OST_NEXP1 -le $OST_NEXP2 ] && error "client not evicted"
 	return 0
 }
-run_test 26 "evict dead exports"
+run_test 26a "evict dead exports"
 
 test_26b() {      # bug 10140 - evict dead exports by pinger
 	client_df
 	zconf_mount `hostname` $MOUNT2 || error "Failed to mount $MOUNT2"
-	MDS_FILE=$LPROC/mds/${mds_svc}/num_exports
-        MDS_NEXP1="`do_facet mds cat $MDS_FILE | cut -d' ' -f2`"
-	OST_FILE=$LPROC/obdfilter/${ost1_svc}/num_exports
-        OST_NEXP1="`do_facet ost1 cat $OST_FILE | cut -d' ' -f2`"
+	MDS_FILE=mds.${mds_svc}.num_exports
+        MDS_NEXP1="`do_facet mds lctl get_param -n $MDS_FILE | cut -d' ' -f2`"
+	OST_FILE=obdfilter.${ost1_svc}.num_exports
+        OST_NEXP1="`do_facet ost1 lctl get_param -n $OST_FILE | cut -d' ' -f2`"
 	echo starting with $OST_NEXP1 OST and $MDS_NEXP1 MDS exports
 	#force umount a client; exports should get evicted
 	zconf_umount `hostname` $MOUNT2 -f
-   # evictor takes up to 2.25x to evict.  But if there's a 
-   # race to start the evictor from various obds, the loser
-   # might have to wait for the next ping.
-   echo Waiting for $(($TIMEOUT * 4)) secs
-   sleep $(($TIMEOUT * 4))
-        OST_NEXP2="`do_facet ost1 cat $OST_FILE | cut -d' ' -f2`"
-        MDS_NEXP2="`do_facet mds cat $MDS_FILE | cut -d' ' -f2`"
+	# evictor takes PING_EVICT_TIMEOUT + 3 * PING_INTERVAL to evict.  
+        # But if there's a race to start the evictor from various obds, 
+        # the loser might have to wait for the next ping.
+	echo Waiting for $(($TIMEOUT * 3)) secs
+	sleep $(($TIMEOUT * 3))
+        OST_NEXP2="`do_facet ost1 lctl get_param -n $OST_FILE | cut -d' ' -f2`"
+        MDS_NEXP2="`do_facet mds lctl get_param -n $MDS_FILE | cut -d' ' -f2`"
 	echo ending with $OST_NEXP2 OST and $MDS_NEXP2 MDS exports
         [ $OST_NEXP1 -le $OST_NEXP2 ] && error "client not evicted from OST"
         [ $MDS_NEXP1 -le $MDS_NEXP2 ] && error "client not evicted from MDS"
@@ -674,7 +716,7 @@ test_50() {
 	echo writemany returned $rc
 	#these may fail because of eviction due to slow AST response.
 	debugrestore
-	return $rc
+	[ $rc -eq 0 ] || error_ignore 13652 "writemany returned rc $rc" || true
 }
 run_test 50 "failover MDS under load"
 
@@ -703,7 +745,7 @@ test_51() {
 	wait $CLIENT_PID 
 	rc=$?
 	echo writemany returned $rc
-	return $rc
+	[ $rc -eq 0 ] || error_ignore 13652 "writemany returned rc $rc" || true
 }
 run_test 51 "failover MDS during recovery"
 
@@ -834,10 +876,8 @@ run_test 56 "do not allow reconnect to busy exports"
 
 test_57_helper() {
         # no oscs means no client or mdt 
-        while [ -e $LPROC/osc ]; do
-                for f in `find $LPROC -type f`; do 
-                        cat $f > /dev/null 2>&1
-                done
+        while lctl get_param osc.*.* > /dev/null 2>&1; do
+                : # loop until proc file is removed
         done
 }
 
@@ -874,13 +914,18 @@ run_test 58 "Eviction in the middle of open RPC reply processing"
 
 test_59() { # bug 10589
 	zconf_mount `hostname` $MOUNT2 || error "Failed to mount $MOUNT2"
+	echo $DIR2 | grep -q $MOUNT2 || error "DIR2 is not set properly: $DIR2"
 	sysctl -w lustre.fail_loc=0x311
-	writes=`dd if=/dev/zero of=$DIR2/$tfile count=1 2>&1 | awk 'BEGIN { FS="+" } /out/ {print $1}'`
+	writes=$(LANG=C dd if=/dev/zero of=$DIR2/$tfile count=1 2>&1)
+	[ $? = 0 ] || error "dd write failed"
+	writes=$(echo $writes | awk  -F '+' '/out/ {print $1}')
 	sysctl -w lustre.fail_loc=0
 	sync
-	zconf_umount `hostname` $DIR2 -f
-	reads=`dd if=$DIR/$tfile of=/dev/null 2>&1 | awk 'BEGIN { FS="+" } /in/ {print $1}'`
-	[ $reads -eq $writes ] || error "read" $reads "blocks, must be" $writes
+	zconf_umount `hostname` $MOUNT2 -f
+	reads=$(LANG=C dd if=$DIR/$tfile of=/dev/null 2>&1)
+	[ $? = 0 ] || error "dd read failed"
+	reads=$(echo $reads | awk -F '+' '/in/ {print $1}')
+	[ "$reads" -eq "$writes" ] || error "read" $reads "blocks, must be" $writes
 }
 run_test 59 "Read cancel race on client eviction"
 
