@@ -20,12 +20,17 @@ GRANT_CHECK_LIST=${GRANT_CHECK_LIST:-""}
 # bug number: 
 ALWAYS_EXCEPT="$REPLAY_SINGLE_EXCEPT"
 
+#                                                     63 min  7 min  AT AT AT AT"
+[ "$SLOW" = "no" ] && EXCEPT_SLOW="1 2 3 4 6 6b 12 16 44      44b    65 66 67 68"
+
 build_test_filter
 
 cleanup_and_setup_lustre
 
 mkdir -p $DIR
 
+rm -rf $DIR/[df][0-9]*
+
 test_0() {
     replay_barrier mds
     fail mds
@@ -188,7 +193,6 @@ run_test 5 "|x| 220 open(O_CREAT)"
 
 test_6() {
     replay_barrier mds
-    mkdir $DIR/$tdir
     mcreate $DIR/$tdir/$tfile
     fail mds
     $CHECKSTAT -t dir $DIR/$tdir || return 1
@@ -207,7 +211,6 @@ test_6b() {
 run_test 6b "|X| rmdir"
 
 test_7() {
-    mkdir $DIR/$tdir
     replay_barrier mds
     mcreate $DIR/$tdir/$tfile
     fail mds
@@ -219,9 +222,8 @@ run_test 7 "mkdir |X| contained create"
 
 test_8() {
     replay_barrier mds
-    multiop $DIR/$tfile mo_c &
+    multiop_bg_pause $DIR/$tfile mo_c || return 4
     MULTIPID=$!
-    sleep 1
     fail mds
     ls $DIR/$tfile
     $CHECKSTAT -t file $DIR/$tfile || return 1
@@ -279,10 +281,8 @@ run_test 11 "create open write rename |X| create-old-name read"
 
 test_12() {
     mcreate $DIR/$tfile 
-    multiop $DIR/$tfile o_tSc &
+    multiop_bg_pause $DIR/$tfile o_tSc || return 3
     pid=$!
-    # give multiop a chance to open
-    sleep 1
     rm -f $DIR/$tfile
     replay_barrier mds
     kill -USR1 $pid
@@ -299,10 +299,8 @@ run_test 12 "open, unlink |X| close"
 #        a regular open a failure    
 test_13() {
     mcreate $DIR/$tfile 
-    multiop $DIR/$tfile O_wc &
+    multiop_bg_pause $DIR/$tfile O_wc || return 3
     pid=$!
-    # give multiop a chance to open
-    sleep 1 
     chmod 0 $DIR/$tfile
     $CHECKSTAT -p 0 $DIR/$tfile
     replay_barrier mds
@@ -316,10 +314,8 @@ test_13() {
 run_test 13 "open chmod 0 |x| write close"
 
 test_14() {
-    multiop $DIR/$tfile O_tSc &
+    multiop_bg_pause $DIR/$tfile O_tSc || return 4
     pid=$!
-    # give multiop a chance to open
-    sleep 1 
     rm -f $DIR/$tfile
     replay_barrier mds
     kill -USR1 $pid || return 1
@@ -332,10 +328,8 @@ test_14() {
 run_test 14 "open(O_CREAT), unlink |X| close"
 
 test_15() {
-    multiop $DIR/$tfile O_tSc &
+    multiop_bg_pause $DIR/$tfile O_tSc || return 5
     pid=$!
-    # give multiop a chance to open
-    sleep 1 
     rm -f $DIR/$tfile
     replay_barrier mds
     touch $DIR/g11 || return 1
@@ -364,10 +358,8 @@ run_test 16 "|X| open(O_CREAT), unlink, touch new,  unlink new"
 
 test_17() {
     replay_barrier mds
-    multiop $DIR/$tfile O_c &
+    multiop_bg_pause $DIR/$tfile O_c || return 4
     pid=$!
-    # give multiop a chance to open
-    sleep 1 
     fail mds
     kill -USR1 $pid || return 1
     wait $pid || return 2
@@ -378,10 +370,8 @@ run_test 17 "|X| open(O_CREAT), |replay| close"
 
 test_18() {
     replay_barrier mds
-    multiop $DIR/$tfile O_tSc &
+    multiop_bg_pause $DIR/$tfile O_tSc || return 8
     pid=$!
-    # give multiop a chance to open
-    sleep 1 
     rm -f $DIR/$tfile
     touch $DIR/$tfile-2 || return 1
     echo "pid: $pid will close"
@@ -413,10 +403,8 @@ run_test 19 "|X| mcreate, open, write, rename "
 
 test_20() {
     replay_barrier mds
-    multiop $DIR/$tfile O_tSc &
+    multiop_bg_pause $DIR/$tfile O_tSc || return 3
     pid=$!
-    # give multiop a chance to open
-    sleep 1 
     rm -f $DIR/$tfile
 
     fail mds
@@ -474,10 +462,8 @@ run_test 20c "check that client eviction does not affect file content"
 
 test_21() {
     replay_barrier mds
-    multiop $DIR/$tfile O_tSc &
+    multiop_bg_pause $DIR/$tfile O_tSc || return 5
     pid=$!
-    # give multiop a chance to open
-    sleep 1 
     rm -f $DIR/$tfile
     touch $DIR/g11 || return 1
 
@@ -491,10 +477,8 @@ test_21() {
 run_test 21 "|X| open(O_CREAT), unlink touch new, replay, close (test mds_cleanup_orphans)"
 
 test_22() {
-    multiop $DIR/$tfile O_tSc &
+    multiop_bg_pause $DIR/$tfile O_tSc || return 3
     pid=$!
-    # give multiop a chance to open
-    sleep 1 
 
     replay_barrier mds
     rm -f $DIR/$tfile
@@ -508,10 +492,8 @@ test_22() {
 run_test 22 "open(O_CREAT), |X| unlink, replay, close (test mds_cleanup_orphans)"
 
 test_23() {
-    multiop $DIR/$tfile O_tSc &
+    multiop_bg_pause $DIR/$tfile O_tSc || return 5
     pid=$!
-    # give multiop a chance to open
-    sleep 1 
 
     replay_barrier mds
     rm -f $DIR/$tfile
@@ -527,10 +509,8 @@ test_23() {
 run_test 23 "open(O_CREAT), |X| unlink touch new, replay, close (test mds_cleanup_orphans)"
 
 test_24() {
-    multiop $DIR/$tfile O_tSc &
+    multiop_bg_pause $DIR/$tfile O_tSc || return 3
     pid=$!
-    # give multiop a chance to open
-    sleep 1 
 
     replay_barrier mds
     fail mds
@@ -543,10 +523,8 @@ test_24() {
 run_test 24 "open(O_CREAT), replay, unlink, close (test mds_cleanup_orphans)"
 
 test_25() {
-    multiop $DIR/$tfile O_tSc &
+    multiop_bg_pause $DIR/$tfile O_tSc || return 3
     pid=$!
-    # give multiop a chance to open
-    sleep 1 
     rm -f $DIR/$tfile
 
     replay_barrier mds
@@ -560,12 +538,10 @@ run_test 25 "open(O_CREAT), unlink, replay, close (test mds_cleanup_orphans)"
 
 test_26() {
     replay_barrier mds
-    multiop $DIR/$tfile-1 O_tSc &
+    multiop_bg_pause $DIR/$tfile-1 O_tSc || return 5
     pid1=$!
-    multiop $DIR/$tfile-2 O_tSc &
+    multiop_bg_pause $DIR/$tfile-2 O_tSc || return 6
     pid2=$!
-    # give multiop a chance to open
-    sleep 1 
     rm -f $DIR/$tfile-1
     rm -f $DIR/$tfile-2
     kill -USR1 $pid2
@@ -582,12 +558,10 @@ run_test 26 "|X| open(O_CREAT), unlink two, close one, replay, close one (test m
 
 test_27() {
     replay_barrier mds
-    multiop $DIR/$tfile-1 O_tSc &
+    multiop_bg_pause $DIR/$tfile-1 O_tSc || return 5
     pid1=$!
-    multiop $DIR/$tfile-2 O_tSc &
+    multiop_bg_pause $DIR/$tfile-2 O_tSc || return 6
     pid2=$!
-    # give multiop a chance to open
-    sleep 1 
     rm -f $DIR/$tfile-1
     rm -f $DIR/$tfile-2
 
@@ -603,12 +577,10 @@ test_27() {
 run_test 27 "|X| open(O_CREAT), unlink two, replay, close two (test mds_cleanup_orphans)"
 
 test_28() {
-    multiop $DIR/$tfile-1 O_tSc &
+    multiop_bg_pause $DIR/$tfile-1 O_tSc || return 5
     pid1=$!
-    multiop $DIR/$tfile-2 O_tSc &
+    multiop_bg_pause $DIR/$tfile-2 O_tSc || return 6
     pid2=$!
-    # give multiop a chance to open
-    sleep 1 
     replay_barrier mds
     rm -f $DIR/$tfile-1
     rm -f $DIR/$tfile-2
@@ -625,12 +597,10 @@ test_28() {
 run_test 28 "open(O_CREAT), |X| unlink two, close one, replay, close one (test mds_cleanup_orphans)"
 
 test_29() {
-    multiop $DIR/$tfile-1 O_tSc &
+    multiop_bg_pause $DIR/$tfile-1 O_tSc || return 5
     pid1=$!
-    multiop $DIR/$tfile-2 O_tSc &
+    multiop_bg_pause $DIR/$tfile-2 O_tSc || return 6
     pid2=$!
-    # give multiop a chance to open
-    sleep 1 
     replay_barrier mds
     rm -f $DIR/$tfile-1
     rm -f $DIR/$tfile-2
@@ -647,12 +617,10 @@ test_29() {
 run_test 29 "open(O_CREAT), |X| unlink two, replay, close two (test mds_cleanup_orphans)"
 
 test_30() {
-    multiop $DIR/$tfile-1 O_tSc &
+    multiop_bg_pause $DIR/$tfile-1 O_tSc || return 5
     pid1=$!
-    multiop $DIR/$tfile-2 O_tSc &
+    multiop_bg_pause $DIR/$tfile-2 O_tSc || return 6
     pid2=$!
-    # give multiop a chance to open
-    sleep 1 
     rm -f $DIR/$tfile-1
     rm -f $DIR/$tfile-2
 
@@ -669,12 +637,10 @@ test_30() {
 run_test 30 "open(O_CREAT) two, unlink two, replay, close two (test mds_cleanup_orphans)"
 
 test_31() {
-    multiop $DIR/$tfile-1 O_tSc &
+    multiop_bg_pause $DIR/$tfile-1 O_tSc || return 5
     pid1=$!
-    multiop $DIR/$tfile-2 O_tSc &
+    multiop_bg_pause $DIR/$tfile-2 O_tSc || return 6
     pid2=$!
-    # give multiop a chance to open
-    sleep 1 
     rm -f $DIR/$tfile-1
 
     replay_barrier mds
@@ -693,12 +659,10 @@ run_test 31 "open(O_CREAT) two, unlink one, |X| unlink one, close two (test mds_
 # tests for bug 2104; completion without crashing is success.  The close is
 # stale, but we always return 0 for close, so the app never sees it.
 test_32() {
-    multiop $DIR/$tfile O_c &
+    multiop_bg_pause $DIR/$tfile O_c || return 2
     pid1=$!
-    multiop $DIR/$tfile O_c &
+    multiop_bg_pause $DIR/$tfile O_c || return 3
     pid2=$!
-    # give multiop a chance to open
-    sleep 1
     mds_evict_client
     df $MOUNT || sleep 1 && df $MOUNT || return 1
     kill -USR1 $pid1
@@ -721,10 +685,8 @@ test_33() {
 run_test 33 "abort recovery before client does replay"
 
 test_34() {
-    multiop $DIR/$tfile O_c &
+    multiop_bg_pause $DIR/$tfile O_c || return 2
     pid=$!
-    # give multiop a chance to open
-    sleep 1 
     rm -f $DIR/$tfile
 
     replay_barrier mds
@@ -771,10 +733,8 @@ run_test 36 "don't resend cancel"
 # directory orphans can't be unlinked from PENDING directory
 test_37() {
     rmdir $DIR/$tfile 2>/dev/null
-    multiop $DIR/$tfile dD_c &
+    multiop_bg_pause $DIR/$tfile dD_c || return 2
     pid=$!
-    # give multiop a chance to open
-    sleep 1 
     rmdir $DIR/$tfile
 
     replay_barrier mds
@@ -865,7 +825,7 @@ test_41() {
 
     local f=$MOUNT/$tfile
     # make sure the start of the file is ost1
-    lfs setstripe $f $((128 * 1024)) 0 0 
+    lfs setstripe $f -s $((128 * 1024)) -i 0 
     do_facet client dd if=/dev/zero of=$f bs=4k count=1 || return 3
     cancel_lru_locks osc
     # fail ost2 and read from ost1
@@ -914,15 +874,28 @@ test_43() { # bug 2530
 run_test 43 "mds osc import failure during recovery; don't LBUG"
 
 test_44() {
+    local at_max_saved=0
+
     mdcdev=`awk '/-mdc-/ {print $1}' $LPROC/devices`
     [ "$mdcdev" ] || exit 2
+
+    # adaptive timeouts slow this way down
+    if at_is_valid && at_is_enabled; then
+        at_max_saved=$(at_max_get mds)
+        at_max_set 40 mds
+    fi
+
     for i in `seq 1 10`; do
+	echo "$i of 10 ($(date +%s))"
+	do_facet mds "grep service $LPROC/mdt/MDS/mds/timeouts"
 	#define OBD_FAIL_TGT_CONN_RACE     0x701
 	do_facet mds "sysctl -w lustre.fail_loc=0x80000701"
 	$LCTL --device $mdcdev recover
 	df $MOUNT
     done
+
     do_facet mds "sysctl -w lustre.fail_loc=0"
+    [ $at_max_saved -ne 0 ] && at_max_set $at_max_saved mds
     return 0
 }
 run_test 44 "race in target handle connect"
@@ -932,6 +905,7 @@ test_44b() {
     [ "$mdcdev" ] || exit 2
     for i in `seq 1 10`; do
 	echo "$i of 10 ($(date +%s))"
+	do_facet mds "grep service $LPROC/mdt/MDS/mds/timeouts"
 	#define OBD_FAIL_TGT_DELAY_RECONNECT 0x704
 	do_facet mds "sysctl -w lustre.fail_loc=0x80000704"
 	$LCTL --device $mdcdev recover
@@ -948,9 +922,8 @@ test_45() {
     [ "$mdcdev" ] || exit 2
     $LCTL --device $mdcdev recover
 
-    multiop $DIR/$tfile O_c &
+    multiop_bg_pause $DIR/$tfile O_c || return 1
     pid=$!
-    sleep 1
 
     # This will cause the CLOSE to fail before even 
     # allocating a reply buffer
@@ -1019,9 +992,10 @@ test_48() {
 run_test 48 "MDS->OSC failure during precreate cleanup (2824)"
 
 test_50() {
-    local oscdev=`grep ${ost1_svc}-osc- $LPROC/devices | awk '{print $1}'`
+    local oscdev=`do_facet mds grep \'${ost1_svc}-osc \' $LPROC/devices | awk '{print $1}' | head -1`
     [ "$oscdev" ] || return 1
-    $LCTL --device $oscdev recover &&  $LCTL --device $oscdev recover
+    do_facet mds $LCTL --device $oscdev recover || return 2 
+    do_facet mds $LCTL --device $oscdev recover || return 3 
     # give the mds_lov_sync threads a chance to run
     sleep 5
 }
@@ -1043,8 +1017,235 @@ test_52() {
 }
 run_test 52 "time out lock replay (3764)"
 
-#b_cray 53 "|X| open request and close reply while two MDC requests in flight"
-#b_cray 54 "|X| open request and close reply while two MDC requests in flight"
+# bug 3462 - simultaneous MDC requests
+test_53a() {
+    mkdir -p $DIR/${tdir}-1
+    mkdir -p $DIR/${tdir}-2
+    multiop $DIR/${tdir}-1/f O_c &
+    close_pid=$!
+    # give multiop a chance to open
+    sleep 1
+
+    #define OBD_FAIL_MDS_CLOSE_NET 0x115
+    do_facet mds "sysctl -w lustre.fail_loc=0x80000115"
+    kill -USR1 $close_pid
+    cancel_lru_locks MDC # force the close
+    do_facet mds "sysctl -w lustre.fail_loc=0"
+    mcreate $DIR/${tdir}-2/f || return 1
+    
+    # close should still be here
+    [ -d /proc/$close_pid ] || return 2
+    replay_barrier_nodf mds
+    fail mds
+    wait $close_pid || return 3
+
+    $CHECKSTAT -t file $DIR/${tdir}-1/f || return 4
+    $CHECKSTAT -t file $DIR/${tdir}-2/f || return 5
+    rm -rf $DIR/${tdir}-*
+}
+run_test 53a "|X| close request while two MDC requests in flight"
+
+test_53b() {
+    mkdir -p $DIR/$tdir-1
+    mkdir -p $DIR/$tdir-2
+    multiop $DIR/$tdir-1/f O_c &
+    close_pid=$!
+
+    #define OBD_FAIL_MDS_REINT_NET 0x107
+    do_facet mds "sysctl -w lustre.fail_loc=0x80000107"
+    mcreate $DIR/${tdir}-2/f &
+    open_pid=$!
+    sleep 1
+
+    do_facet mds "sysctl -w lustre.fail_loc=0"
+    kill -USR1 $close_pid
+    cancel_lru_locks MDC # force the close
+    wait $close_pid || return 1
+    # open should still be here
+    [ -d /proc/$open_pid ] || return 2
+
+    replay_barrier_nodf mds
+    fail mds
+    wait $open_pid || return 3
+
+    $CHECKSTAT -t file $DIR/${tdir}-1/f || return 4
+    $CHECKSTAT -t file $DIR/${tdir}-2/f || return 5
+    rm -rf $DIR/${tdir}-*
+}
+run_test 53b "|X| open request while two MDC requests in flight"
+
+test_53c() {
+    mkdir -p $DIR/${tdir}-1
+    mkdir -p $DIR/${tdir}-2
+    multiop $DIR/${tdir}-1/f O_c &
+    close_pid=$!
+
+    do_facet mds "sysctl -w lustre.fail_loc=0x80000107"
+    mcreate $DIR/${tdir}-2/f &
+    open_pid=$!
+    sleep 1
+
+    do_facet mds "sysctl -w lustre.fail_loc=0x80000115"
+    kill -USR1 $close_pid
+    cancel_lru_locks MDC  # force the close
+
+    replay_barrier_nodf mds
+    fail_nodf mds
+    wait $open_pid || return 1
+    sleep 2
+    # close should be gone
+    [ -d /proc/$close_pid ] && return 2
+    do_facet mds "sysctl -w lustre.fail_loc=0"
+
+    $CHECKSTAT -t file $DIR/${tdir}-1/f || return 3
+    $CHECKSTAT -t file $DIR/${tdir}-2/f || return 4
+    rm -rf $DIR/${tdir}-*
+}
+run_test 53c "|X| open request and close request while two MDC requests in flight"
+
+test_53d() {
+    mkdir -p $DIR/${tdir}-1
+    mkdir -p $DIR/${tdir}-2
+    multiop $DIR/${tdir}-1/f O_c &
+    close_pid=$!
+    # give multiop a chance to open
+    sleep 1
+
+    # define OBD_FAIL_MDS_CLOSE_NET_REP 0X138    
+    do_facet mds "sysctl -w lustre.fail_loc=0x8000013b"
+    kill -USR1 $close_pid
+    cancel_lru_locks MDC  # force the close
+    do_facet mds "sysctl -w lustre.fail_loc=0"
+    mcreate $DIR/${tdir}-2/f || return 1
+    
+    # close should still be here
+    [ -d /proc/$close_pid ] || return 2
+    replay_barrier_nodf mds
+    fail mds
+    wait $close_pid || return 3
+
+    $CHECKSTAT -t file $DIR/${tdir}-1/f || return 4
+    $CHECKSTAT -t file $DIR/${tdir}-2/f || return 5
+    rm -rf $DIR/${tdir}-*
+}
+run_test 53d "|X| close reply while two MDC requests in flight"
+
+test_53e() {
+    mkdir -p $DIR/$tdir-1
+    mkdir -p $DIR/$tdir-2
+    multiop $DIR/$tdir-1/f O_c &
+    close_pid=$!
+
+    #define OBD_FAIL_MDS_REINT_NET_REP       0x119
+    do_facet mds "sysctl -w lustre.fail_loc=0x80000119"
+    mcreate $DIR/${tdir}-2/f &
+    open_pid=$!
+    sleep 1
+    
+    do_facet mds "sysctl -w lustre.fail_loc=0"
+    kill -USR1 $close_pid
+    cancel_lru_locks MDC  # force the close
+    wait $close_pid || return 1
+    # open should still be here
+    [ -d /proc/$open_pid ] || return 2
+    
+    replay_barrier_nodf mds
+    fail mds
+    wait $open_pid || return 3
+
+    $CHECKSTAT -t file $DIR/${tdir}-1/f || return 4
+    $CHECKSTAT -t file $DIR/${tdir}-2/f || return 5
+    rm -rf $DIR/${tdir}-*
+}
+run_test 53e "|X| open reply while two MDC requests in flight"
+
+test_53f() {
+        mkdir -p $DIR/${tdir}-1
+        mkdir -p $DIR/${tdir}-2
+        multiop $DIR/${tdir}-1/f O_c &
+        close_pid=$!
+
+        do_facet mds "sysctl -w lustre.fail_loc=0x80000119"
+        mcreate $DIR/${tdir}-2/f &
+        open_pid=$!
+        sleep 1
+
+        do_facet mds "sysctl -w lustre.fail_loc=0x8000013b"
+        kill -USR1 $close_pid
+        cancel_lru_locks MDC
+
+        replay_barrier_nodf mds
+        fail_nodf mds
+        wait $open_pid || return 1
+        sleep 2
+        #close should be gone
+        [ -d /proc/$close_pid ] && return 2
+        do_facet mds "sysctl -w lustre.fail_loc=0"
+
+        $CHECKSTAT -t file $DIR/${tdir}-1/f || return 3
+        $CHECKSTAT -t file $DIR/${tdir}-2/f || return 4
+        rm -rf $DIR/${tdir}-*
+}
+run_test 53f "|X| open reply and close reply while two MDC requests in flight"
+
+test_53g() {
+        mkdir -p $DIR/${tdir}-1
+        mkdir -p $DIR/${tdir}-2
+        multiop $DIR/${tdir}-1/f O_c &
+        close_pid=$!
+
+        do_facet mds "sysctl -w lustre.fail_loc=0x80000119"
+        mcreate $DIR/${tdir}-2/f &
+        open_pid=$!
+        sleep 1
+
+        do_facet mds "sysctl -w lustre.fail_loc=0x80000115"
+        kill -USR1 $close_pid
+        cancel_lru_locks MDC # force the close
+
+        do_facet mds "sysctl -w lustre.fail_loc=0"
+        replay_barrier_nodf mds
+        fail_nodf mds
+        wait $open_pid || return 1
+        sleep 2
+        # close should be gone
+        [ -d /proc/$close_pid ] && return 2
+
+        $CHECKSTAT -t file $DIR/${tdir}-1/f || return 3
+        $CHECKSTAT -t file $DIR/${tdir}-2/f || return 4
+        rm -rf $DIR/${tdir}-*
+}
+run_test 53g "|X| drop open reply and close request while close and open are both in flight"
+
+test_53h() {
+    mkdir -p $DIR/${tdir}-1
+    mkdir -p $DIR/${tdir}-2
+    multiop $DIR/${tdir}-1/f O_c &
+    close_pid=$!
+
+    do_facet mds "sysctl -w lustre.fail_loc=0x80000107"
+    mcreate $DIR/${tdir}-2/f &
+    open_pid=$!
+    sleep 1
+    
+    do_facet mds "sysctl -w lustre.fail_loc=0x8000013b"
+    kill -USR1 $close_pid
+    cancel_lru_locks MDC  # force the close
+    sleep 1
+
+    replay_barrier_nodf mds
+    fail_nodf mds
+    wait $open_pid || return 1
+    sleep 2
+    # close should be gone
+    [ -d /proc/$close_pid ] && return 2
+    do_facet mds "sysctl -w lustre.fail_loc=0"
+
+    $CHECKSTAT -t file $DIR/${tdir}-1/f || return 3
+    $CHECKSTAT -t file $DIR/${tdir}-2/f || return 4
+    rm -rf $DIR/${tdir}-*
+}
+run_test 53h "|X| open request and close reply while two MDC requests in flight"
 
 #b3761 ASSERTION(hash != 0) failed
 test_55() {
@@ -1087,12 +1288,11 @@ run_test 57 "test recovery from llog for setattr op"
 test_58() {
 #define OBD_FAIL_MDS_OST_SETATTR       0x12c
     do_facet mds "sysctl -w lustre.fail_loc=0x8000012c"
-    mkdir $DIR/$tdir
     createmany -o $DIR/$tdir/$tfile-%d 2500
     replay_barrier mds
     fail mds
     sleep 2
-    $CHECKSTAT -t file $DIR/$tdir/$tfile-* || return 1
+    $CHECKSTAT -t file $DIR/$tdir/$tfile-* >/dev/null || return 1
     do_facet mds "sysctl -w lustre.fail_loc=0x0"
     unlinkmany $DIR/$tdir/$tfile-%d 2500
     rmdir $DIR/$tdir
@@ -1102,7 +1302,6 @@ run_test 58 "test recovery from llog for setattr op (test llog_gen_rec)"
 # log_commit_thread vs filter_destroy race used to lead to import use after free
 # bug 11658
 test_59() {
-    mkdir $DIR/$tdir
     createmany -o $DIR/$tdir/$tfile-%d 200
     sync
     unlinkmany $DIR/$tdir/$tfile-%d 200
@@ -1119,7 +1318,6 @@ run_test 59 "test log_commit_thread vs filter_destroy race"
 # race between add unlink llog vs cat log init in post_recovery (only for b1_6)
 # bug 12086: should no oops and No ctxt error for this test
 test_60() {
-    mkdir $DIR/$tdir
     createmany -o $DIR/$tdir/$tfile-%d 200
     replay_barrier mds
     unlinkmany $DIR/$tdir/$tfile-%d 0 100
@@ -1132,7 +1330,6 @@ run_test 60 "test llog post recovery init vs llog unlink"
 
 #test race  llog recovery thread vs llog cleanup
 test_61a() {
-    mkdir $DIR/$tdir
     createmany -o $DIR/$tdir/$tfile-%d 800
     replay_barrier ost1 
 #   OBD_FAIL_OST_LLOG_RECOVERY_TIMEOUT 0x221 
@@ -1170,6 +1367,201 @@ test_61c() {
 }
 run_test 61c "test race mds llog sync vs llog cleanup"
 
+#Adaptive Timeouts (bug 3055)
+AT_MAX_SET=0
+
+at_start()
+{
+    at_is_valid || skip "AT env is invalid"
+
+    if ! at_is_enabled; then
+        echo "AT is disabled, enable it by force temporarily"
+        at_max_set 600 mds ost client
+        AT_MAX_SET=1
+    fi
+
+    if [ -z "$ATOLDBASE" ]; then
+	local at_history=$(do_facet mds "find /sys/ -name at_history")
+	[ -z "$at_history" ] && skip "missing /sys/.../at_history " && return 1
+	ATOLDBASE=$(do_facet mds "cat $at_history")
+        # speed up the timebase so we can check decreasing AT
+	do_facet mds "echo 8 >> $at_history"
+	do_facet ost1 "echo 8 >> $at_history"
+    fi
+}
+
+test_65a() #bug 3055
+{
+    at_start || return 0
+    $LCTL dk > /dev/null
+    debugsave
+    sysctl -w lnet.debug="+other"
+    # slow down a request
+    do_facet mds sysctl -w lustre.fail_val=30000
+#define OBD_FAIL_PTLRPC_PAUSE_REQ        0x50a
+    do_facet mds sysctl -w lustre.fail_loc=0x8000050a
+    createmany -o $DIR/$tfile 10 > /dev/null
+    unlinkmany $DIR/$tfile 10 > /dev/null
+    # check for log message
+    $LCTL dk | grep "Early reply #" || error "No early reply" 
+    # client should show 30s estimates
+    grep portal $LPROC/mdc/${FSNAME}-MDT0000-mdc-*/timeouts
+    sleep 9
+    grep portal $LPROC/mdc/${FSNAME}-MDT0000-mdc-*/timeouts
+}
+run_test 65a "AT: verify early replies"
+
+test_65b() #bug 3055
+{
+    at_start || return 0
+    # turn on D_ADAPTTO
+    debugsave
+    sysctl -w lnet.debug="+other"
+    $LCTL dk > /dev/null
+    # slow down bulk i/o
+    do_facet ost1 sysctl -w lustre.fail_val=30
+#define OBD_FAIL_OST_BRW_PAUSE_PACK      0x224
+    do_facet ost1 sysctl -w lustre.fail_loc=0x224
+
+    rm -f $DIR/$tfile
+    lfs setstripe $DIR/$tfile --index=0 --count=1
+    # force some real bulk transfer
+    multiop $DIR/$tfile oO_CREAT:O_RDWR:O_SYNC:w4096c
+
+    do_facet ost1 sysctl -w lustre.fail_loc=0
+    # check for log message
+    $LCTL dk | grep "Early reply #" || error "No early reply"
+    debugrestore
+    # client should show 30s estimates
+    grep portal $LPROC/osc/${FSNAME}-OST0000-osc-*/timeouts
+}
+run_test 65b "AT: verify early replies on packed reply / bulk"
+
+test_66a() #bug 3055
+{
+    at_start || return 0
+    grep "portal 12" $LPROC/mdc/${FSNAME}-MDT0000-mdc-*/timeouts
+    # adjust 5s at a time so no early reply is sent (within deadline)
+    do_facet mds "sysctl -w lustre.fail_val=5000"
+#define OBD_FAIL_PTLRPC_PAUSE_REQ        0x50a
+    do_facet mds "sysctl -w lustre.fail_loc=0x8000050a"
+    createmany -o $DIR/$tfile 20 > /dev/null
+    unlinkmany $DIR/$tfile 20 > /dev/null
+    grep "portal 12" $LPROC/mdc/${FSNAME}-MDT0000-mdc-*/timeouts
+    do_facet mds "sysctl -w lustre.fail_val=10000"
+    do_facet mds "sysctl -w lustre.fail_loc=0x8000050a"
+    createmany -o $DIR/$tfile 20 > /dev/null
+    unlinkmany $DIR/$tfile 20 > /dev/null
+    grep "portal 12" $LPROC/mdc/${FSNAME}-MDT0000-mdc-*/timeouts
+    do_facet mds "sysctl -w lustre.fail_loc=0"
+    sleep 9
+    createmany -o $DIR/$tfile 20 > /dev/null
+    unlinkmany $DIR/$tfile 20 > /dev/null
+    grep portal $LPROC/mdc/${FSNAME}-MDT0000-mdc-*/timeouts | grep "portal 12"
+    CUR=$(awk '/portal 12/ {print $5}' $LPROC/mdc/${FSNAME}-MDT0000-mdc-*/timeouts)
+    WORST=$(awk '/portal 12/ {print $7}' $LPROC/mdc/${FSNAME}-MDT0000-mdc-*/timeouts)
+    echo "Current MDT timeout $CUR, worst $WORST"
+    [ $CUR -lt $WORST ] || error "Current $CUR should be less than worst $WORST" 
+}
+run_test 66a "AT: verify MDT service time adjusts with no early replies"
+
+test_66b() #bug 3055
+{
+    at_start || return 0
+    ORIG=$(awk '/network/ {print $4}' $LPROC/mdc/${FSNAME}-*/timeouts)
+    sysctl -w lustre.fail_val=$(($ORIG + 5))
+#define OBD_FAIL_PTLRPC_PAUSE_REP      0x50c
+    sysctl -w lustre.fail_loc=0x50c
+    ls $DIR/$tfile > /dev/null 2>&1
+    sysctl -w lustre.fail_loc=0
+    CUR=$(awk '/network/ {print $4}' $LPROC/mdc/${FSNAME}-*/timeouts)
+    WORST=$(awk '/network/ {print $6}' $LPROC/mdc/${FSNAME}-*/timeouts)
+    echo "network timeout orig $ORIG, cur $CUR, worst $WORST"
+    [ $WORST -gt $ORIG ] || error "Worst $WORST should be worse than orig $ORIG" 
+}
+run_test 66b "AT: verify net latency adjusts"
+
+test_67a() #bug 3055
+{
+    at_start || return 0
+    CONN1=$(awk '/_connect/ {total+=$2} END {print total}' $LPROC/osc/*/stats)
+    # sleeping threads may drive values above this
+    do_facet ost1 "sysctl -w lustre.fail_val=400"
+#define OBD_FAIL_PTLRPC_PAUSE_REQ    0x50a
+    do_facet ost1 "sysctl -w lustre.fail_loc=0x50a"
+    createmany -o $DIR/$tfile 20 > /dev/null
+    unlinkmany $DIR/$tfile 20 > /dev/null
+    do_facet ost1 "sysctl -w lustre.fail_loc=0"
+    CONN2=$(awk '/_connect/ {total+=$2} END {print total}' $LPROC/osc/*/stats)
+    ATTEMPTS=$(($CONN2 - $CONN1))
+    echo "$ATTEMPTS osc reconnect attemps on gradual slow"
+    [ $ATTEMPTS -gt 0 ] && error_ignore 13721 "AT should have prevented reconnect"
+    return 0
+}
+run_test 67a "AT: verify slow request processing doesn't induce reconnects"
+
+test_67b() #bug 3055
+{
+    at_start || return 0
+    CONN1=$(awk '/_connect/ {total+=$2} END {print total}' $LPROC/osc/*/stats)
+#define OBD_FAIL_OST_PAUSE_CREATE        0x223
+    do_facet ost1 "sysctl -w lustre.fail_val=20000"
+    do_facet ost1 "sysctl -w lustre.fail_loc=0x80000223"
+    cp /etc/profile $DIR/$tfile || error "cp failed"
+    client_reconnect
+    cat $LPROC/ost/OSS/ost_create/timeouts
+    log "phase 2"
+    CONN2=$(awk '/_connect/ {total+=$2} END {print total}' $LPROC/osc/*/stats)
+    ATTEMPTS=$(($CONN2 - $CONN1))
+    echo "$ATTEMPTS osc reconnect attemps on instant slow"
+    # do it again; should not timeout
+    do_facet ost1 "sysctl -w lustre.fail_loc=0x80000223"
+    cp /etc/profile $DIR/$tfile || error "cp failed"
+    do_facet ost1 "sysctl -w lustre.fail_loc=0"
+    client_reconnect
+    cat $LPROC/ost/OSS/ost_create/timeouts
+    CONN3=$(awk '/_connect/ {total+=$2} END {print total}' $LPROC/osc/*/stats)
+    ATTEMPTS=$(($CONN3 - $CONN2))
+    echo "$ATTEMPTS osc reconnect attemps on 2nd slow"
+    [ $ATTEMPTS -gt 0 ] && error "AT should have prevented reconnect"
+    return 0
+}
+run_test 67b "AT: verify instant slowdown doesn't induce reconnects"
+
+test_68 () #bug 13813
+{
+    at_start || return 0
+    local ldlm_enqueue_min=$(find /sys -name ldlm_enqueue_min)
+    [ -z "$ldlm_enqueue_min" ] && skip "missing /sys/.../ldlm_enqueue_min" && return 0
+    local ENQ_MIN=$(cat $ldlm_enqueue_min)
+    echo $TIMEOUT >> $ldlm_enqueue_min
+    rm -f $DIR/${tfile}_[1-2]
+    lfs setstripe $DIR/$tfile --index=0 --count=1
+#define OBD_FAIL_LDLM_PAUSE_CANCEL       0x312
+    sysctl -w lustre.fail_val=$(($TIMEOUT - 1))
+    sysctl -w lustre.fail_loc=0x80000312
+    cp /etc/profile $DIR/${tfile}_1 || error "1st cp failed $?"
+    sysctl -w lustre.fail_val=$((TIMEOUT * 3 / 2))
+    sysctl -w lustre.fail_loc=0x80000312
+    cp /etc/profile $DIR/${tfile}_2 || error "2nd cp failed $?"
+    sysctl -w lustre.fail_loc=0
+    echo $ENQ_MIN >> $ldlm_enqueue_min
+    return 0
+}
+run_test 68 "AT: verify slowing locks"
+
+if [ -n "$ATOLDBASE" ]; then
+    at_history=$(do_facet mds "find /sys/ -name at_history")
+    do_facet mds "echo $ATOLDBASE >> $at_history" || true
+    do_facet ost1 "echo $ATOLDBASE >> $at_history" || true
+fi
+
+if [ $AT_MAX_SET -ne 0 ]; then
+    echo "restore AT status to be disabled"
+    at_max_set 0 mds ost client
+fi
+
+# end of AT tests includes above lines
 
 equals_msg `basename $0`: test complete, cleaning up
 check_and_cleanup_lustre
