@@ -3,11 +3,12 @@
 set -e
 
 ONLY=${ONLY:-"$*"}
-# bug number for skipped test:  3192 12652
-ALWAYS_EXCEPT="                 14b  14c   $SANITYN_EXCEPT"
+# bug number for skipped test:  3192 12652  9977
+ALWAYS_EXCEPT="                 14b  14c    28   $SANITYN_EXCEPT"
 # UPDATE THE COMMENT ABOVE WITH BUG NUMBERS WHEN CHANGING ALWAYS_EXCEPT!
 
-[ "$SLOW" = "no" ] && EXCEPT="$EXCEPT 16"
+# bug number for skipped test:                                                    12652 12652
+grep -q 'Enterprise Server 10' /etc/SuSE-release && ALWAYS_EXCEPT="$ALWAYS_EXCEPT 11    14" || true
 
 # Tests that fail on uml
 [ "$UML" = "true" ] && EXCEPT="$EXCEPT 7"
@@ -30,14 +31,6 @@ MOUNT_2=${MOUNT_2:-"yes"}
 CHECK_GRANT=${CHECK_GRANT:-"yes"}
 GRANT_CHECK_LIST=${GRANT_CHECK_LIST:-""}
 
-if [ $UID -ne 0 ]; then
-	RUNAS_ID="$UID"
-	RUNAS=""
-else
-	RUNAS_ID=${RUNAS_ID:-500}
-	RUNAS=${RUNAS:-"runas -u $RUNAS_ID"}
-fi
-
 SAVE_PWD=$PWD
 
 export NAME=${NAME:-local}
@@ -49,6 +42,8 @@ SETUP=${SETUP:-:}
 init_test_env $@
 . ${CONFIG:=$LUSTRE/tests/cfg/$NAME.sh}
 
+[ "$SLOW" = "no" ] && EXCEPT_SLOW="12 16"
+
 SANITYLOG=${TESTSUITELOG:-$TMP/$(basename $0 .sh).log}
 FAIL_ON_ERROR=false
 
@@ -62,11 +57,16 @@ LPROC=/proc/fs/lustre
 check_and_setup_lustre
 
 LPROC=/proc/fs/lustre
-LOVNAME=`cat $LPROC/llite/*/lov/common_name | tail -n 1`
-OSTCOUNT=`cat $LPROC/lov/$LOVNAME/numobd`
+LOVNAME=`lctl get_param -n llite.*.lov.common_name | tail -n 1`
+OSTCOUNT=`lctl get_param -n lov.$LOVNAME.numobd`
 
 rm -rf $DIR1/[df][0-9]* $DIR1/lnk
 
+# $RUNAS_ID may get set incorrectly somewhere else
+[ $UID -eq 0 -a $RUNAS_ID -eq 0 ] && error "\$RUNAS_ID set to 0, but \$UID is also 0!"
+
+check_runas_id $RUNAS_ID $RUNAS
+
 build_test_filter
 
 test_1a() {
@@ -158,6 +158,9 @@ test_6() {
 run_test 6 "remove of open file on other node =================="
 
 test_7() {
+	# run_one creates uniq $tdir (bug 13798)
+	# opendirunlink failes if it exists
+	rmdir $DIR1/$tdir || true
 	opendirunlink $DIR1/$tdir $DIR2/$tdir || \
 		error "opendirunlink $DIR1/$tdir $DIR2/$tdir"
 }
@@ -214,9 +217,8 @@ run_test 10b "write of file with sub-page size on multiple mounts "
 
 test_11() {
 	mkdir $DIR1/d11
-	multiop $DIR1/d11/f O_c &
+	multiop_bg_pause $DIR1/d11/f O_c || return 1
 	MULTIPID=$!
-	sleep 0.200s
 	cp -p /bin/ls $DIR1/d11/f
 	$DIR2/d11/f
 	RC=$?
@@ -227,7 +229,7 @@ test_11() {
 run_test 11 "execution of file opened for write should return error ===="
 
 test_12() {
-       sh lockorder.sh
+       DIR=$DIR DIR2=$DIR2 sh lockorder.sh
 }
 run_test 12 "test lock ordering (link, stat, unlink) ==========="
 
@@ -260,12 +262,11 @@ run_test 14 "execution of file open for write returns -ETXTBSY ="
 test_14a() {
         mkdir -p $DIR1/d14
 	cp -p `which multiop` $DIR1/d14/multiop || error "cp failed"
-        $DIR1/d14/multiop $TMP/test14.junk O_c &
-        MULTIPID=$!
-        sleep 1
+        MULTIOP_PROG=$DIR1/d14/multiop multiop_bg_pause $TMP/test14.junk O_c || return 1
+        MULTIOP_PID=$!
         multiop $DIR2/d14/multiop Oc && error "expected error, got success"
-        kill -USR1 $MULTIPID || return 2
-        wait $MULTIPID || return 3
+        kill -USR1 $MULTIOP_PID || return 2
+        wait $MULTIOP_PID || return 3
         rm $TMP/test14.junk $DIR1/d14/multiop || error "removing multiop"
 }
 run_test 14a "open(RDWR) of executing file returns -ETXTBSY ===="
@@ -273,13 +274,12 @@ run_test 14a "open(RDWR) of executing file returns -ETXTBSY ===="
 test_14b() { # bug 3192, 7040
         mkdir -p $DIR1/d14
 	cp -p `which multiop` $DIR1/d14/multiop || error "cp failed"
-        $DIR1/d14/multiop $TMP/test14.junk O_c &
-        MULTIPID=$!
-        sleep 1
-        truncate $DIR2/d14/multiop 0 && kill -9 $MULTIPID && \
+        MULTIOP_PROG=$DIR1/d14/multiop multiop_bg_pause $TMP/test14.junk O_c || return 1
+        MULTIOP_PID=$!
+        truncate $DIR2/d14/multiop 0 && kill -9 $MULTIOP_PID && \
 		error "expected truncate error, got success"
-        kill -USR1 $MULTIPID || return 2
-        wait $MULTIPID || return 3
+        kill -USR1 $MULTIOP_PID || return 2
+        wait $MULTIOP_PID || return 3
 	cmp `which multiop` $DIR1/d14/multiop || error "binary changed"
 	rm $TMP/test14.junk $DIR1/d14/multiop || error "removing multiop"
 }
@@ -288,12 +288,11 @@ run_test 14b "truncate of executing file returns -ETXTBSY ======"
 test_14c() { # bug 3430, 7040
 	mkdir -p $DIR1/d14
 	cp -p `which multiop` $DIR1/d14/multiop || error "cp failed"
-	$DIR1/d14/multiop $TMP/test14.junk O_c &
-	MULTIPID=$!
-	sleep 1
+	MULTIOP_PROG=$DIR1/d14/multiop multiop_bg_pause $TMP/test14.junk O_c || return 1
+        MULTIOP_PID=$!
 	cp /etc/hosts $DIR2/d14/multiop && error "expected error, got success"
-	kill -USR1 $MULTIPID || return 2
-	wait $MULTIPID || return 3
+	kill -USR1 $MULTIOP_PID || return 2
+	wait $MULTIOP_PID || return 3
 	cmp `which multiop` $DIR1/d14/multiop || error "binary changed"
 	rm $TMP/test14.junk $DIR1/d14/multiop || error "removing multiop"
 }
@@ -302,13 +301,12 @@ run_test 14c "open(O_TRUNC) of executing file return -ETXTBSY =="
 test_14d() { # bug 10921
 	mkdir -p $DIR1/d14
 	cp -p `which multiop` $DIR1/d14/multiop || error "cp failed"
-	$DIR1/d14/multiop $TMP/test14.junk O_c &
-	MULTIPID=$!
-	sleep 1
+	MULTIOP_PROG=$DIR1/d14/multiop multiop_bg_pause $TMP/test14.junk O_c || return 1
+        MULTIOP_PID=$!
 	log chmod
 	chmod 600 $DIR1/d14/multiop || error "chmod failed"
-	kill -USR1 $MULTIPID || return 2
-	wait $MULTIPID || return 3
+	kill -USR1 $MULTIOP_PID || return 2
+	wait $MULTIOP_PID || return 3
 	cmp `which multiop` $DIR1/d14/multiop || error "binary changed"
 	rm $TMP/test14.junk $DIR1/d14/multiop || error "removing multiop"
 }
@@ -324,7 +322,7 @@ run_test 15 "test out-of-space with multiple writers ==========="
 
 test_16() {
 	rm -f $MOUNT1/fsxfile
-	lfs setstripe $MOUNT1/fsxfile 0 -1 -1 # b=10919
+	lfs setstripe $MOUNT1/fsxfile -c -1 # b=10919
 	fsx -c 50 -p 100 -N 2500 -l $((SIZE * 256)) -S 0 $MOUNT1/fsxfile $MOUNT2/fsxfile
 }
 run_test 16 "2500 iterations of dual-mount fsx ================="
@@ -438,10 +436,9 @@ test_23() { # Bug 5972
 	time1=`date +%s`	
 	sleep 2
 	
-	multiop $DIR1/f23 or20_c &
+	multiop_bg_pause $DIR1/f23 or20_c || return 1
 	MULTIPID=$!
 
-	sleep 2
 	time2=`stat -c "%X" $DIR2/f23`
 
 	if (( $time2 <= $time1 )); then
@@ -473,10 +470,9 @@ test_24() {
 run_test 24 "lfs df [-ih] [path] test ========================="
 
 test_25() {
-	[ `cat $LPROC/mdc/*-mdc-*/connect_flags | grep -c acl` -lt 2 ] && \
+	[ `lctl get_param -n mdc.*-mdc-*.connect_flags | grep -c acl` -lt 2 ] && \
 	    skip "must have acl, skipping" && return
 
-	mkdir $DIR1/$tdir || error "mkdir $DIR1/$tdir"
 	touch $DIR1/$tdir/f1 || error "touch $DIR1/$tdir/f1"
 	chmod 0755 $DIR1/$tdir/f1 || error "chmod 0755 $DIR1/$tdir/f1"
 
@@ -529,7 +525,7 @@ test_27() {
 	sleep 1
 	dd if=/dev/zero of=$DIR1/$tfile bs=8k conv=notrunc count=1 seek=0
 	log "dd 3 finished"
-	echo > $LPROC/ldlm/dump_namespaces
+	lctl set_param -n ldlm.dump_namespaces ""
 	wait $DD1_PID $DD2_PID
 	[ $? -ne 0 ] && lctl dk $TMP/debug || true
 }
@@ -538,7 +534,7 @@ run_test 27 "align non-overlapping extent locks from request ==="
 test_28() { # bug 9977
 	ostID=`$LCTL dl | awk '/-osc-|OSC.*MNT/ { ost++; if (ost == 2) { print $1 } }'`
 
-	lfs setstripe $DIR1/$tfile 1048576 0 2
+	lfs setstripe $DIR1/$tfile -s 1048576 -i 0 -c 2
 	tOBJID=`lfs getstripe $DIR1/$tfile | awk '/^[[:space:]]+1/ {print $2}'`
 	dd if=/dev/zero of=$DIR1/$tfile bs=1024k count=2
 
@@ -571,8 +567,6 @@ test_29() { # bug 10999
 #run_test 29 "lock put race between glimpse and enqueue ========="
 
 test_30() { #bug #11110
-    rm -rf $DIR1/$tdir
-    mkdir -p $DIR1/$tdir
     cp -f /bin/bash $DIR1/$tdir/bash
     /bin/sh -c 'sleep 1; rm -f $DIR2/$tdir/bash; cp /bin/bash $DIR2/$tdir' &
     err=$($DIR1/$tdir/bash -c 'sleep 2; openfile -f O_RDONLY /proc/$$/exe >& /dev/null; echo $?')
@@ -583,6 +577,18 @@ test_30() { #bug #11110
 
 run_test 30 "recreate file race ========="
 
+test_31() {
+        mkdir -p $DIR1/$tdir || error "Creating dir $DIR1/$tdir"
+        writes=`LANG=C dd if=/dev/zero of=$DIR/$tdir/$tfile count=1 2>&1 |
+                awk 'BEGIN { FS="+" } /out/ {print $1}'`
+        #define OBD_FAIL_LDLM_CANCEL_BL_CB_RACE   0x314
+        sysctl -w lustre.fail_loc=0x314
+        reads=`LANG=C dd if=$DIR2/$tdir/$tfile of=/dev/null 2>&1 |
+               awk 'BEGIN { FS="+" } /in/ {print $1}'`
+        [ $reads -eq $writes ] || error "read" $reads "blocks, must be" $writes
+}
+run_test 31 "voluntary cancel / blocking ast race=============="
+
 log "cleanup: ======================================================"
 
 check_and_cleanup_lustre
