@@ -573,6 +573,12 @@ h2gm () {
     fi
 }
 
+h2name_or_ip() {
+    if [ "$1" = "client" -o "$1" = "'*'" ]; then echo \'*\'; else
+        echo $1"@$2"
+    fi
+}
+
 h2ptl() {
    if [ "$1" = "client" -o "$1" = "'*'" ]; then echo \'*\'; else
        ID=`xtprocadmin -n $1 2>/dev/null | egrep -v 'NID' | awk '{print $1}'`
@@ -612,6 +618,11 @@ h2openib() {
 }
 declare -fx h2openib
 
+h2o2ib() {
+    h2name_or_ip "$1" "o2ib"
+}
+declare -fx h2o2ib
+
 facet_host() {
     local facet=$1
     varname=${facet}_HOST
