@@ -119,6 +119,14 @@ command_t cmdlist[] = {
          "usage: conf_param <target.keyword=val> ...\n"},
         {"local_param", jt_lcfg_param, 0, "set a temporary, local param\n"
          "usage: local_param <target.keyword=val> ...\n"},
+        {"get_param", jt_lcfg_getparam, 0, "get the Lustre or LNET parameter\n"
+         "usage: get_param [-n] path/to/param/file \n"
+         "Get the value of Lustre or LNET parameter from the specified path\n"
+         "Use '-n' to disable printing of the key name when printing values."},
+        {"set_param", jt_lcfg_setparam, 0, "set the Lustre or LNET parameter\n"
+         "usage: set_param [-n] path/to/param/file value\n"
+         "Set the value of the Lustre or LNET parameter at the specified path\n"
+         "Use '-n' to disable printing of the key name when printing values."},
 
         /* Debug commands */
         {"==== debugging control ===", jt_noop, 0, "debug"},
